/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef RUNSTRATEGYELEMENT_H
#define RUNSTRATEGYELEMENT_H

#include "StrategyElement.h"

#include <boost/random/mersenne_twister.hpp>
#include <boost/variant/static_visitor.hpp>

namespace tlp {
class PluginProgress;
class Graph;
class DataSet;
}

/**
 * @brief The runElement class is used to execute a strategy element
 */
class runStrategyElement : public boost::static_visitor<bool> {

    tlp::PluginProgress* _pp;
    tlp::Graph* _g;
    tlp::DataSet* _ds;
    const bool _isCond;

    bool run_strat(const StrategyElement::vector_strategyelts &cond, const bool isCond);
    tlp::BooleanProperty* run_focus_operation(const StrategyElement::setPosBan_datastruct &ds, const bool isBan);
    void FromOp(tlp::BooleanProperty &check, tlp::BooleanProperty *prop_old);
    tlp::BooleanProperty* FocusOperation(const bool isBan, tlp::Graph *new_model);

    static boost::mt19937 gen; //random number generator
    //le générateur est static pour éviter d'en créer un nouveau à chaque appel
    unsigned random_number(const StrategyElement::ppick_datastruct &) const;

public:
    runStrategyElement(tlp::PluginProgress *pp, tlp::Graph *g, tlp::DataSet *ds, const bool isCond);

    /**
     * @brief operator () To execute a All() or a One()
     * @param
     * @return true if the rule has been applied successfully
     */
    bool operator()(const StrategyElement::oneall_datastruct &ds );

    /**
     * @brief operator () Execute all the elements of the vector given in parameter
     * @param v the vector containing all elements to execute
     * @return true if all elements have been applied successfully
     */
    bool operator()(const StrategyElement::vector_strategyelts &v);
    /**
     * @brief operator () Execute an "orelse" or a "while" statement
     * @param ifds The datastructure which contains the if to execute
     * @return true if the statement was applied successfully (no failure)
     */
    bool operator()(const StrategyElement::orelse_while_datastruct &ifds);
    /**
     * @brief operator () Execute a "if" statement
     * @param ifds The datastructure which contains the if to execute
     * @return true if the statement was applied successfully (no failure)
     */
    bool operator()(const StrategyElement::if_datastruct &ifds);

    /**
     * @brief operator () Execute a "ppick" statement
     * @param ds the datastructure of the ppick
     * @return  true if the statement has been applied successfully
     */
    bool operator()(const StrategyElement::ppick_datastruct &ds);

    /**
     * @brief operator () Execute a "repeat" statement
     * @param ds The "repeat" datastructure
     * @return  true if the statement has been applied successfully
     */
    bool operator()(const StrategyElement::repeat_ds &ds);

    /**
     * @brief operator () The strategy element is FAIL or ID
     * @param pu
     * @return true if ID, false if FAIL
     */
    bool operator()(const StrategyElement::idfail &pu);

    /**
     * @brief operator () TO execute a "setBan", "setPos" or a "IsEmpty" statement
     * @param fd the datastructure to use
     * @return true if the statement has been applied successfully
     */
    bool operator()(const StrategyElement::focus_datastruct &fd);

};


#endif // RUNSTRATEGYELEMENT_H
