/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "StrategyElement.h"
#include "RunStrategyElement.h"

#include <tulip/Graph.h>

using namespace std;
using namespace boost;
using namespace boost::fusion;
using namespace tlp;

// Id/Fail
StrategyElement::StrategyElement(const idfail op):data(op){
}

// All/One
StrategyElement::StrategyElement(const StrategyElement::one_all_datastruct &rules, const rule_operators op):data(make_pair(rules, op)) {

}

//setPos, setBan, isEmpty
StrategyElement::StrategyElement(const setPosBan_datastruct &d, const strat_focus_op op):data(make_pair(d, op)){
}

//ppick
StrategyElement::StrategyElement(const ppick_datastruct &vect):data(vect) {
}

//for
StrategyElement::StrategyElement(const for_datastruct &p) {
    //recopie le contenu de vect num (p.first) fois
    cerr << "__FOR__" << endl;
    vector_strategyelts v;
    for(unsigned i=0;i<p.first;++i)
        std::copy(p.second.begin(), p.second.end(), std::back_inserter(v));
    data = v;
}

//while, orelse
StrategyElement::StrategyElement(const orelse_while_datastruct &v):data(v) {
}

//if
StrategyElement::StrategyElement(const if_datastruct &v):data(v) {

}

//try(S) equiv. à (S)orelse(Id)
//not(s) equiv. à if(S)then(Fail)else(Id)
//repeat(S) est implémenté en natif
StrategyElement::StrategyElement(const vector_strategyelts &ds, const syntactic_sugar s){
    //strat ID
    vector_strategyelts id;
    id.push_back(new StrategyElement(ID));

    //strat Fail
    vector_strategyelts fail;
    fail.push_back(new StrategyElement(FAIL));

    switch(s) {
    case TRY:
    {
        //fabrication du orelse
#ifndef NDEBUG
        cerr << "__TRY__: rewritten to an ORELSE" << endl;
#endif
        orelse_while_datastruct ods;
        at_c<0>(ods) = ds;
        at_c<1>(ods) = id;
        at_c<2>(ods) = ORELSE;
        data = ods;
    }
        break;
    case NOT:
    {
        if_datastruct ifds;
#ifndef NDEBUG
        cerr << "__NOT__: rewritten to an IF" << endl;
#endif
        at_c<0>(ifds) = ds;
        at_c<1>(ifds) = fail;
        at_c<2>(ifds) = id;
        data=ifds;
    }
        break;
    case REPEAT:
    {
        repeat_ds rds;
        at_c<0>(rds) = ds;
        at_c<1>(rds) = REPEAT;
        data = rds;
    }
        break;
    }
}

///////////////////////////////
bool StrategyElement::run(PluginProgress *pp, Graph *g, DataSet *dataSet, const bool isCond) const {
    if(!isCond)
        g->push(false); //on push le graphe avant chaque étape de la stratégie pour annuler si besoin sauf si condition (sera supprimée lors du test de la condition)
    //execute strategy element
    runStrategyElement visitor(pp, g, dataSet, isCond);
    return data.apply_visitor(visitor);
}
