#include <tulip/ImportModule.h>
#include <tulip/Graph.h>
#include <tulip/ConnectedTest.h>
#include <tulip/StringCollection.h>
#include <tulip/LayoutProperty.h>

#include <portgraph/PorgyTlpGraphStructure.h>
#include <portgraph/PortGraphModel.h>

using namespace tlp;
using namespace std;

namespace {

const char * paramHelp[] = {
    // nodes
    HTML_HELP_OPEN() \
    HTML_HELP_DEF( "type", "unsigned int" ) \
    HTML_HELP_DEF( "default", "5" ) \
    HTML_HELP_BODY() \
    "Number of nodes in the final graph." \
    HTML_HELP_CLOSE(),

    // edges
    HTML_HELP_OPEN() \
    HTML_HELP_DEF( "type", "unsigned int" ) \
    HTML_HELP_DEF( "default", "9" ) \
    HTML_HELP_BODY() \
    "Number of edges in the final graph." \
    HTML_HELP_CLOSE(),

    // connected
    HTML_HELP_OPEN() \
    HTML_HELP_DEF( "type", "boolean" ) \
    HTML_HELP_DEF( "default", "true" ) \
    HTML_HELP_BODY() \
    "True if the graph has to be connected." \
    HTML_HELP_CLOSE(),
};
}

class random_simple_graph : public ImportModule {
public:
    PLUGININFORMATION("Random Simple PortGraph","Pinaud","19/09/2013","Imports a new randomly generated simple graph (portgraph)","1.1","Porgy")
    random_simple_graph(tlp::PluginContext* context):ImportModule(context) {
        addInParameter<unsigned int>("nodes",paramHelp[0],"5");
        addInParameter<unsigned int>("edges",paramHelp[1],"9");
        addInParameter<bool>("Connected",paramHelp[2],"true");

        addDependency("Random Simple Graph", "1.0");
        addDependency(PorgyConstants::REDRAW_GRAPH, "1.0");
    }

    bool importGraph() {
        Graph *g = tlp::importGraph("Random Simple Graph", *dataSet, pluginProgress, graph);

        bool connected=false;
        dataSet->get("Connected", connected);

        if(connected) {
            std::vector<tlp::edge> edges;
            tlp::ConnectedTest::makeConnected(graph, edges);
        }

        if(g!=NULL) {
            Observable::holdObservers();

            PorgyTlpGraphStructure::Graph2PortGraph(graph, pluginProgress);

            //layout new graph
            DataSet dSet;
            string errMsg;
            StringCollection algo(LAYOUT_ALGO);
            algo.setCurrent("GEM (Frick)");
            dSet.set("Layout", algo);
            LayoutProperty *layout = graph->getProperty<LayoutProperty>("viewLayout");
            return graph->applyPropertyAlgorithm(PorgyConstants::REDRAW_GRAPH, layout, errMsg,
                                                 pluginProgress, &dSet);

            Observable::unholdObservers();
            return true;
        }
        else {
            cerr << "problem when importing a new graph" << endl;
            return false;
        }
    }
};

PLUGIN(random_simple_graph)
