#include <tulip/ImportModule.h>
#include <tulip/Graph.h>
#include <tulip/StringCollection.h>
#include <tulip/LayoutProperty.h>

#include <portgraph/PorgyTlpGraphStructure.h>
#include <portgraph/PortGraphModel.h>

using namespace tlp;
using namespace std;

namespace {

const char * paramHelp[] = {
    // minsize
    HTML_HELP_OPEN() \
    HTML_HELP_DEF( "type", "int" ) \
    HTML_HELP_DEF( "default", "10" ) \
    HTML_HELP_BODY() \
    "Minimal number of nodes in the tree." \
    HTML_HELP_CLOSE(),
    // maxsize
    HTML_HELP_OPEN() \
    HTML_HELP_DEF( "type", "int" ) \
    HTML_HELP_DEF( "default", "100" ) \
    HTML_HELP_BODY() \
    "Maximal number of nodes in the tree." \
    HTML_HELP_CLOSE(),
    // maxdegree
    HTML_HELP_OPEN() \
    HTML_HELP_DEF( "type", "int" ) \
    HTML_HELP_DEF( "default", "5" ) \
    HTML_HELP_BODY() \
    "Maximal degree of the nodes." \
    HTML_HELP_CLOSE(),
    // tree layout
    HTML_HELP_OPEN() \
    HTML_HELP_DEF( "type", "bool" ) \
    HTML_HELP_DEF( "default", "false" ) \
    HTML_HELP_BODY() \
    "If true, the generated tree is drawn with a tree layout algorithm." \
    HTML_HELP_CLOSE(),
};
}

class random_tree : public ImportModule {
public:
    PLUGININFORMATION("Random Tree","Pinaud","21/08/2013","Imports a new randomly generated tree (portgraph)","1.1","Porgy")
    random_tree(tlp::PluginContext* context):ImportModule(context) {
        addInParameter<unsigned>("Minimum size",paramHelp[0],"10");
        addInParameter<unsigned>("Maximum size",paramHelp[1],"100");
        addInParameter<unsigned>("Maximal node's degree",paramHelp[2],"5");
        addInParameter<bool>("Apply a tree layout",paramHelp[3],"false");
        addDependency("Random General Tree", "1.1");
        addDependency(PorgyConstants::REDRAW_GRAPH, "1.0");
    }

    bool importGraph() {
        Graph *g = tlp::importGraph("Random General Tree", *dataSet, pluginProgress, graph);
        if(g!=NULL) {
            Observable::holdObservers();

            PorgyTlpGraphStructure::Graph2PortGraph(graph, pluginProgress);

            //layout new graph
            bool layout=false;
            if(dataSet!=NULL) {
                dataSet->get("Apply a tree layout", layout);
            }
            if(layout) {
                // apply Tree Leaf
                DataSet dSet;
                string errMsg;
                StringCollection algo(LAYOUT_ALGO);
                algo.setCurrent("Tree Leaf");
                dSet.set("Layout", algo);
                LayoutProperty *layout = graph->getProperty<LayoutProperty>("viewLayout");
                return graph->applyPropertyAlgorithm(PorgyConstants::REDRAW_GRAPH, layout, errMsg,
                                                     pluginProgress, &dSet);
            }
            Observable::unholdObservers();
            return true;
        }
        else {
            cerr << "problem when importing a new graph" << endl;
            return false;
        }
    }
};

PLUGIN(random_tree)
