/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "InteractorPortGraphAddEdgeComponent.h"

#include <QMouseEvent>

#include <tulip/View.h>
#include <tulip/StringProperty.h>
#include <tulip/GlMainWidget.h>
#include <tulip/GlGraphComposite.h>
#include <tulip/Observable.h>

#include <portgraph/PorgyTlpGraphStructure.h>
#include <portgraph/PortGraphRule.h>
#include <portgraph/Bridge.h>

using namespace tlp;
using namespace std;

PortGraphAddEdgeComponent::PortGraphAddEdgeComponent() {

}

void PortGraphAddEdgeComponent::addLink(QObject *widget, const tlp::node src, const tlp::node target) {
    GlMainWidget *glMainWidget = static_cast<GlMainWidget *>(widget);
    Graph * g = glMainWidget->getScene()->getGlGraphComposite()->getInputData()->getGraph();
    IntegerProperty *side = PortGraphRule::getSideProperty(g);
    IntegerProperty *arity = g->getProperty<IntegerProperty>(PorgyConstants::ARITY);
    LayoutProperty* mLayout = g->getProperty<LayoutProperty>("viewLayout");
    Observable::holdObservers();
    if(!PorgyTlpGraphStructure::isRuleGraph(g)) {
        MouseEdgeBuilder::addLink(widget, src, target);
    }
    else if((PortGraphRule::isMember(src, g, PorgyConstants::SIDE_LEFT))&&(PortGraphRule::isMember(target, g, PorgyConstants::SIDE_RIGHT))) {        
        setBends(vector<Coord>());
        PortGraphRule pgr(g);
        Bridge *b = pgr.getBridge();
        b->addPort(src, target); //add one node and 3 edges to the graph
        b->draw(g->getProperty<SizeProperty>("viewSize"), g->getProperty<ColorProperty>("viewColor"), mLayout, g->getProperty<ColorProperty>("viewBorderColor"));
        b->setPositions(mLayout);
        StringProperty *state = g->getProperty<StringProperty>(PorgyConstants::STATE);
        state->setNodeValue(src, PorgyConstants::STATE_NOTUSED);
        state->setNodeValue(target, PorgyConstants::STATE_NOTUSED);
    }
    else if((PortGraphRule::isMember(src, g, PorgyConstants::SIDE_LEFT))&&(PortGraphRule::isMember(target, g, PorgyConstants::SIDE_LEFT))) {
        edge e = g->addEdge(src, target);
        side->setEdgeValue(e, PorgyConstants::SIDE_LEFT);
        arity->setNodeValue(src, arity->getNodeValue(src)+1);
        arity->setNodeValue(target, arity->getNodeValue(target)+1);
        mLayout->setEdgeValue(e, bends());

    }
    else if((PortGraphRule::isMember(src, g, PorgyConstants::SIDE_RIGHT))&&(PortGraphRule::isMember(target, g, PorgyConstants::SIDE_RIGHT))) {
        edge e = g->addEdge(src, target);
        side->setEdgeValue(e, PorgyConstants::SIDE_RIGHT);
        arity->setNodeValue(src, arity->getNodeValue(src)+1);
        arity->setNodeValue(target, arity->getNodeValue(target)+1);
        mLayout->setEdgeValue(e, bends());

    }
    setBends(vector<Coord>());
    Observable::unholdObservers();
}

bool PortGraphAddEdgeComponent::eventFilter(QObject *widget, QEvent *e) {    
    QMouseEvent *qMouseEv = (QMouseEvent *) e;
    GlMainWidget *glMainWidget = static_cast<GlMainWidget *>(widget);
    Graph * _graph = glMainWidget->getScene()->getGlGraphComposite()->getInputData()->getGraph();
    if ((e->type() == QEvent::MouseButtonPress)&&(qMouseEv->button() == Qt::LeftButton)) {
        SelectedEntity selectedEntity;
        glMainWidget->pickNodesEdges(qMouseEv->x(), qMouseEv->y(), selectedEntity, NULL, true, false);
        node n(selectedEntity.getNode());
        if (n.isValid()) {
            if((PortBase::isPort(n, _graph))&&(!Bridge::isBridge(n, _graph))) {
                if(PorgyTlpGraphStructure::isRuleGraph(_graph)) {
                    if(!source().isValid())
                        return MouseEdgeBuilder::eventFilter(widget, e);
                    else if(!((PortGraphRule::isMember(source(), _graph, PorgyConstants::SIDE_RIGHT))&&(PortGraphRule::isMember(n, _graph, PorgyConstants::SIDE_LEFT))))
                        return MouseEdgeBuilder::eventFilter(widget, e);
                    else
                        return false;
                }
                else
                    return MouseEdgeBuilder::eventFilter(widget, e);
            }
            else
                return false;
        }
    }
    else
        return MouseEdgeBuilder::eventFilter(widget, e);

    return MouseEdgeBuilder::eventFilter(widget, e);
}
