/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "InteractorPortGraphDelElementComponent.h"

#include <tulip/GlMainWidget.h>
#include <tulip/MouseInteractors.h>
#include <tulip/ForEach.h>
#include <tulip/GlGraphComposite.h>
#include <tulip/View.h>

#include <portgraph/Bridge.h>
#include <portgraph/PortGraph.h>
#include <portgraph/PorgyTlpGraphStructure.h>
#include <portgraph/PortGraphRule.h>

#include <QMouseEvent>

using namespace tlp;
using namespace std;

PortGraphDelElementComponent::PortGraphDelElementComponent():tulipedelinteractor(new MouseElementDeleter()), started(false) {
}

PortGraphDelElementComponent::~PortGraphDelElementComponent() {	
    delete tulipedelinteractor;
}

void PortGraphDelElementComponent::viewChanged(View *view) {
    if (view==NULL)
        return;
    InteractorComponent::viewChanged(view);
    tulipedelinteractor->setView(view);
    if(PorgyTlpGraphStructure::isRuleGraph(view->graph())) {
        view->graph()->addListener(this);
        view->graph()->addObserver(this);
    }
}

//si on supprime une des arêtes qui vont vers le bridge, il faut supprimer le port du bridge
//le sommet sera supprimé lors de l'appel   update après la suppression des arêtes
void PortGraphDelElementComponent::delEdge(Graph *g, const edge e) {
    const pair<node, node> &ends = g->ends(e);
    if(!started)
        return;
    if(PortGraphRule::isMember(e, g, PorgyConstants::SIDE_BRIDGE)) {
        if(Bridge::isBridge(ends.second, g)) {
            nodetodelete.push_back(ends.second);
        }
        else if(Bridge::isBridge(ends.first, g)) {
            nodetodelete.push_back(ends.first);
        }
    }
    //lhs or rhs
    else if(((PortGraphRule::isMember(ends.first, g, PorgyConstants::SIDE_LEFT))&&(PortGraphRule::isMember(ends.second, g, PorgyConstants::SIDE_LEFT)))||((PortGraphRule::isMember(ends.first, g, PorgyConstants::SIDE_RIGHT))&&(PortGraphRule::isMember(ends.second, g, PorgyConstants::SIDE_RIGHT)))) {
        IntegerProperty *arity = g->getProperty<IntegerProperty>(PorgyConstants::ARITY);
        arity->setNodeValue(ends.first, arity->getNodeValue(ends.first)-1);
        arity->setNodeValue(ends.second, arity->getNodeValue(ends.second)-1);
    }
}


bool PortGraphDelElementComponent::eventFilter(QObject *widget, QEvent *e) {
    QMouseEvent *qMouseEv = (QMouseEvent *) e;
    GlMainWidget *glMainWidget = static_cast<GlMainWidget *>(widget);
    Graph * _graph = glMainWidget->getScene()->getGlGraphComposite()->getInputData()->getGraph();
    if(e->type() == QEvent::MouseMove) {
        SelectedEntity selectedEntity;
        glMainWidget->pickNodesEdges(qMouseEv->x(), qMouseEv->y(), selectedEntity, NULL, true, false);
        node n(selectedEntity.getNode());
        if (n.isValid()&&(Bridge::isBridge(n,_graph)&&(PortNodeBase::isCenter(n,_graph)))) {
            glMainWidget->setCursor(Qt::ArrowCursor);
            return true;
        }
        else {
            return tulipedelinteractor->eventFilter(widget, e);
        }
    }
    else if ((e->type() == QEvent::MouseButtonPress)&&(qMouseEv->button() == Qt::LeftButton)) {
        SelectedEntity selectedEntity;
        if (glMainWidget->pickNodesEdges(qMouseEv->x(), qMouseEv->y(), selectedEntity)) {
            switch(selectedEntity.getEntityType()) {
            case SelectedEntity::EDGE_SELECTED:
            {
                edge ed(selectedEntity.getEdge());
                //Only edges between ports or coming from the bridge can be deleted
                if((PortGraph::isPortGraphEdge(ed, _graph))||(PortGraphRule::isMember(ed, _graph, PorgyConstants::SIDE_BRIDGE)))
                    started=true;
                return tulipedelinteractor->eventFilter(widget, e);
            }
            case SelectedEntity::NODE_SELECTED:
            {
                node n(selectedEntity.getNode());
                if(PortNodeBase::isCenter(n, _graph)&&!Bridge::isBridge(n, _graph)) {
                    started=true;
                    delPortNode(n, _graph);
                    return true;
                }
                else if(PortBase::isPort(n, _graph)&&!Bridge::isBridge(n, _graph)) {
                    node center;
                    forEach(center, _graph->getInOutNodes(n)) {
                        if(PortNodeBase::isCenter(center, _graph)) {
                            started=true;
                            delPortNode(center, _graph);
                            return true;
                        }
                    }
                }
                else if(PortBase::isPort(n, _graph)&&Bridge::isBridge(n, _graph)) {
                    started=true;
                    return tulipedelinteractor->eventFilter(widget, e);
                }
                break;
            }
            default://Do nothing
                break;
            }
        }
    }
    else
        return tulipedelinteractor->eventFilter(widget, e);

    return false;
}


void PortGraphDelElementComponent::treatEvent(const tlp::Event &message){
    const tlp::GraphEvent* gEvt = dynamic_cast<const tlp::GraphEvent*>(&message);
    if(gEvt){
        if(gEvt->getType()==tlp::GraphEvent::TLP_DEL_EDGE){
            delEdge(gEvt->getGraph(),gEvt->getEdge());
        }
    }
}

void PortGraphDelElementComponent::treatEvents(const std::vector<tlp::Event> &){
    started=false;
    if(!nodetodelete.empty()) {
        Graph *g = view()->graph();
        for(vector<node>::const_iterator it2=nodetodelete.begin();it2!=nodetodelete.end();++it2) {
            if((g->isElement(*it2))&&(PortBase::isPort(*it2,g))) {
                view()->graph()->delNode(*it2);
            }
        }
        nodetodelete.clear();
        view()->draw();
    }
}

void PortGraphDelElementComponent::delPortNode(node center, Graph * _graph) {
    node n;
    _graph->push(); // allow to undo
    Observable::holdObservers();
    stableForEach(n, _graph->getInOutNodes(center)) {
        _graph->delNode(n);
    }
    _graph->delNode(center);
    Observable::unholdObservers();
}
