/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <portgraph/PorgyConstants.h>

#include <porgy/PortQt.h>

#include "createPort.h"

#include <tulip/TlpQtTools.h>
#include <tulip/GlyphManager.h>

#include "ui_DialogPort.h"

using namespace std;
using namespace tlp;

CreatePort::CreatePort(QWidget *parent, const bool isRule) : QDialog(parent), ui(new Ui::DialogPort)
{
    init(tlp::colorToQColor(PorgyConstants::DEFAULT_COLOR), isRule);
}

//modif d'un port existant
CreatePort::CreatePort(QWidget *parent, const PortQt &port, const bool isRule) : QDialog(parent), ui(new Ui::DialogPort)
{
    init(port.getColor(), isRule);
    ui->lineEditName->setText(port.getName());
    ui->spinBoxArity->setValue(port.getArity());
    if(port.getState() == PorgyConstants::STATE_NOTUSED.c_str()) {
        ui->CheckBoxNotUsed->setChecked(true);
        ui->PortStateLineEdit->setEnabled(false);
    }
    else
        ui->PortStateLineEdit->setText(port.getState());

    ui->comboBoxGlyph->setCurrentIndex(ui->comboBoxGlyph->findText(tlpStringToQString(GlyphManager::getInst().glyphName(port.getShape()))));
}

CreatePort::~CreatePort() {
    delete ui;
}

void CreatePort::init(const QColor &color, bool isRule) {
    ui->setupUi(this);
    string s;
    std::list<std::string> glyphs = tlp::PluginLister::instance()->availablePlugins<tlp::Glyph>();
    for(std::list<std::string>::iterator it = glyphs.begin() ; it != glyphs.end() ; ++it){
        if(s == "2D - Billboard")       //on ne veut pas du Billboard car ce glyph pose trop de problème dans les métasommets
            continue;
        ui->comboBoxGlyph->addItem(it->c_str());
    }
    ui->ColorPortButton->setColor(color);
    connect(ui->CheckBoxNotUsed, SIGNAL(stateChanged(int)), this, SLOT(radioNotUsed_clicked(int)));
    if(!isRule) {
        ui->spinBoxArity->setEnabled(false);
        ui->labelArity->setEnabled(false);
        ui->CheckBoxNotUsed->setEnabled(false);
    }
}

QString CreatePort::getName() const {
    return ui->lineEditName->text();
}

NodeShape::NodeShapes CreatePort::getGlyph() const {
    return static_cast<NodeShape::NodeShapes>(GlyphManager::getInst().glyphId(QStringToTlpString(ui->comboBoxGlyph->currentText())));
}

QColor CreatePort::getColor() const {
    return ui->ColorPortButton->color();
}

double CreatePort::getArity() const {
    return ui->spinBoxArity->value();
}

bool CreatePort::isNotUsed() const {
    return ui->CheckBoxNotUsed->isChecked();
}

QString CreatePort::getState() const {
    return ui->PortStateLineEdit->text();
}

void CreatePort::radioNotUsed_clicked(int state)
{
    if(state==Qt::Checked)
        ui->PortStateLineEdit->setEnabled(false);
    else
        ui->PortStateLineEdit->setEnabled(true);
}

