/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <tulip/PropertyAlgorithm.h>
#include <tulip/DoubleProperty.h>

#include <portgraph/PortGraphModel.h>
#include <portgraph/PortGraphRule.h>
#include <portgraph/PorgyTlpGraphStructure.h>

using namespace tlp;
using namespace std;

class PortNodeDegreeMetric:public tlp::DoubleAlgorithm {

public:
    PLUGININFORMATION("Portnodes' Degree","Bruno Pinaud","01/09/2009","Alpha","1.0",PorgyConstants::CATEGORY_NAME)

PortNodeDegreeMetric(const PluginContext *context):DoubleAlgorithm(context){
}

bool check(string &errMsg) {
    if(!(PorgyTlpGraphStructure::isModelGraph(graph)||PorgyTlpGraphStructure::isRuleGraph(graph))) {
        errMsg = "Active Tulip graph ("+ graph->getName() + ") is not a Graph nor a rule";
        return false;
    }
    return true;
}

bool run() {

    PortGraph *p = NULL;
    if(PorgyTlpGraphStructure::isModelGraph(graph))
        p = new PortGraphModel(graph);
    else
        p = new PortGraphRule(graph);

        Iterator<PortNode *> *it=p->getPortNodes();
        while(it->hasNext()) {
            PortNode *pn = it->next();
            result->setNodeValue(pn->getCenter(),pn->deg());
            Port::PortPtr port;
            forEach(port, pn->getPorts()) {
                result->setNodeValue(port->getNode(), port->deg());
            }
        }delete it;
    delete p;

    return true;
}
};
PLUGIN(PortNodeDegreeMetric)
