#ifndef HELPWIDGET_H
#define HELPWIDGET_H

#include <QUrl>
#include <QMap>
#include <QString>
#include <QWidget>
#include <QTextBrowser>



namespace Ui {
    class HelpWidget;
}

class QHelpEngine;
class QHelpEngineCore;
class QLineEdit;


/**
 * @brief The QtHelpTextBrowser class is a browser capable to get and display data from Qt help resource files.
 */
class QtHelpTextBrowser : public QTextBrowser
{
    Q_OBJECT

    QHelpEngineCore *_helpEngine;
public:
    explicit QtHelpTextBrowser(QWidget* parent=NULL);

    QVariant loadResource ( int type, const QUrl & name );

    void setHelpEngineCore(QHelpEngineCore* helpEngine);
};



/**
 * @brief The HelpWidget class is a simple widget to display Qt help file.
 */
class HelpWidget : public QWidget
{
    Q_OBJECT


    Ui::HelpWidget *_ui;
    QHelpEngine *_helpEngine;
    QWidget *_indexWidget;
    QLineEdit *_indexFilter;
public:

    enum Target{CurrentTab,NewTab};

    explicit HelpWidget(QWidget *parent = 0);
    ~HelpWidget();
    bool setCollectionFile(const QString& collectionFile);

    /**
     * @brief helpEngine get the current help engine.
     * @return
     */
    QHelpEngine* helpEngine()const{
        return _helpEngine;
    }

    /**
     * @brief showLink print the given url in the widget.
     * @param link
     * @param targetTab
     */
    void showLink(const QUrl & link,Target targetTab);

    QString collectionFileTitle()const;
public slots:
    void goHomePage();
    void goPreviousPage();
    void goNextPage();

private slots:
    void linkActivated ( const QUrl & link, const QString & keyword );
    void linksActivated ( const QMap<QString, QUrl> & links, const QString & keyword );
    void linkActivated ( const QUrl & link );

    void tabCloseRequested ( int index );
    void updateBackwardForwardButtonState();
    void historyChanged();
    void showHideSearchPanel(bool isVisible);


};


#endif // HELPWIDGET_H
