#ifndef LOGLISTWIDGET_H
#define LOGLISTWIDGET_H

#include <QWidget>
#include <QSortFilterProxyModel>
#include <log.h>

namespace Ui {
class LogWidget;
}


Q_DECLARE_FLAGS(LogLevels,Log::Level)

class LogModelSortFilterProxy : public  QSortFilterProxyModel{

    LogLevels _levelsFlags;
public:
    LogModelSortFilterProxy(QObject* parent=NULL);

    LogLevels filterFlags()const{
        return _levelsFlags;
    }

    void setFilterFlag(LogLevels levels){
        _levelsFlags = levels;
        invalidateFilter();
    }

    void sort(int column, Qt::SortOrder order){
        sourceModel()->sort(column,order);
    }

protected:
    bool filterAcceptsRow ( int source_row, const QModelIndex & source_parent ) const;
    bool lessThan ( const QModelIndex & left, const QModelIndex & right ) const;
};

/**
 * @brief The LogWidget class gives a GUI to display and handle LogModel objects.
 */
class LogWidget : public QWidget
{
    Q_OBJECT

    Ui::LogWidget *ui;
    LogModel* _model;
    LogModelSortFilterProxy* _filter;
public:
    explicit LogWidget(QWidget *parent = 0);
    ~LogWidget();
    
    void setLogModel(LogModel* model);
    LogModel* logModel()const;

    void setVisible(bool visible);

private slots:
    void clearList();
    void markSeen();

    void filterToggled(bool toggled);
};

#endif // LOGLISTWIDGET_H
