/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef STRATEGYMANAGEMENTWIDGET_H
#define STRATEGYMANAGEMENTWIDGET_H

#include <QWidget>
#include <QAbstractListModel>
#include "strategymanager.h"

namespace Ui {
    class StrategyManagementWidget;
}

class StrategyManagementWidget : public QWidget
{
    Q_OBJECT

    Ui::StrategyManagementWidget *ui;
    StrategyManager strategies;  

public:
    explicit StrategyManagementWidget(QWidget *parent = 0);
    virtual ~StrategyManagementWidget();
    void setGraph(tlp::Graph *g);

    /**
      * @brief Get strategy management widget data. Allow to restore previously saved state with setData function.
      * @see setData
      **/
//    tlp::DataSet getData()const;
    /**
      * @brief Restore previously saved data.
      **/
    void setData(const tlp::DataSet&);

    bool eventFilter(QObject *obj, QEvent *event);

public slots:

    QString getCurrentStrategyCode()const;
    void addNewEmptyStrategy();
    void addNewStrategy(QString name,QString strategy);
    /**
      * @brief Try to create a strategy from the elements selected in the root trace graph. The name of the new strategy will be the
      **/
    void addStrategyFromSelectedElementsInTraceRoot();

    void importStrategy();
    void importStrategy(const QString& filename);
    void exportCurrentStrategy();
    void exportCurrentStrategy(const QString& filename);
    /**
     * @brief importStrategies treat all the files in the directory as strategies and try to import them.
     * @param dirName the path of the directory to use
     */
    void importStrategies(const QString& dirName);
    /**
     * @brief exportStrategies export all the strategies in the given directory.
     * @param dirName
     */
    void exportStrategies(const QString& dirName);
    void removeCurrentStrategy();
    void selectStrategy(unsigned int index);
    void displayStrategyCode(unsigned int index);
    int currentStrategy()const;

protected:
    void changeEvent(QEvent *e);

private slots:
    void selectedStrategyChanged(const QModelIndex & current, const QModelIndex & previous);

    void executeButtonPressed();

private:
    void exportStrategy(int strategyIndex,const QString& filename);
signals:
    void executeStrategy(QString strategyCode);

};

#endif
