/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef TRACESWIDGET_H
#define TRACESWIDGET_H

#include <QWidget>
#include "tracesitemmodel.h"

namespace Ui {
    class TracesWidget;
}

namespace tlp{
    class Graph;
}

class QModelIndex;
class GraphHierarchyTreeWidget;
class SortFilterProxyModel;

class TracesWidget : public QWidget
{
    Q_OBJECT    

public:
    explicit TracesWidget(QWidget *parent = 0);
    ~TracesWidget();
    void setGraph(tlp::Graph* graph);
    inline tlp::Graph* getGraph()const{return graph;}
    QList<tlp::Graph*> selectedTraces()const;
    bool eventFilter(QObject *, QEvent *);


private:
    Ui::TracesWidget *ui;
    tlp::Graph* graph;
    TracesItemModel *_model;
    SortFilterProxyModel *_proxyModel;


private slots:
    void addNewTrace();
    void deleteTrace();
    void showTrace();
    void ShowTreeContextMenu(const QPoint& point);
    void indexDoubleClicked(const QModelIndex& index);
    void filterTraces(const QString& pattern);

signals:    
    void deleteTrace(tlp::Graph*);
    void showTrace(tlp::Graph*,bool openWithDefaultView);


};

#endif // TRACESWIDGET_H
