#ifndef VIEWMANAGER_H
#define VIEWMANAGER_H

#include <string>

#include <tulip/DataSet.h>

#include <QWidget>
#include <QMap>

namespace tlp{
class Graph;
class View;
class GraphHierarchiesModel;
class TulipProject;
class PluginProgress;
}

namespace Ui{
class ViewManager;
class MDIAreaWorkspace;
class WorkspacePanelImplementation;
}

class ViewManagerImplementation : public QObject{
    Q_OBJECT
public:

    ViewManagerImplementation(QObject* parent=NULL):QObject(parent){}

    virtual ~ViewManagerImplementation(){}
    virtual void setupUi(QWidget* parent)=0;
    virtual void showView(tlp::View *view)=0;    
    virtual void drawAllViews()=0;
    virtual QList<tlp::View*> getAllViews()const=0;
    virtual void saveToProject(tlp::TulipProject* project, QMap<tlp::Graph *, QString> rootIds, tlp::PluginProgress*)=0;
    virtual void restoreFromProject(tlp::TulipProject* project, QMap<QString, tlp::Graph *> rootIds, tlp::PluginProgress*)=0;
    virtual void closeView(tlp::View*)=0;
    virtual void closeAll()=0;
signals:
    void showGraphRequest(tlp::Graph*);
    void currentViewActivated(tlp::View*);
};


/**
 * @brief The ViewManager class is the interface between the perspective and the different kind of workspace
 */
class ViewManager : public QWidget
{
    Q_OBJECT

    Ui::ViewManager* _ui;
    ViewManagerImplementation* _implementation;
    QWidget* _parentWidget;
    tlp::View* _currentView;
public:
    ViewManager(QWidget* parent = NULL);
    ~ViewManager();

    void setImplementation(ViewManagerImplementation* implementation);
    /**
     * @brief createView create a new view
     * @param viewName
     * @param graph
     * @param parameters
     */
    tlp::View* createView(const std::string& viewName, tlp::Graph* graph, const tlp::DataSet& parameters = tlp::DataSet());

    /**
     * @brief currentView return the currently activated view or NULL if there is no view.
     * @return
     */
    tlp::View* currentView()const;

    /**
     * @brief drawAllView
     */
    void drawAllViews();

    QList<tlp::View*> getViewsForGraph(tlp::Graph*)const;
    /**
     * @brief views return all the views.
     * @return
     */
    QList<tlp::View*> views() const;

    void closeView(tlp::View* view);

    void saveToProject(tlp::TulipProject* project, QMap<tlp::Graph *, QString> rootIds, tlp::PluginProgress*);
    void restoreFromProject(tlp::TulipProject* project, QMap<QString, tlp::Graph *> rootIds, tlp::PluginProgress*);

public slots:
    /**
     * @brief closeAllView close all the opened view
     */
    void closeAllView();

signals:
    void showGraphRequest(tlp::Graph*);
    void currentViewChanged(tlp::View*);

private slots:
    void currentViewChangedInternal(tlp::View*);

};

class MDIAreaImplementation: public ViewManagerImplementation{

    Ui::MDIAreaWorkspace* _ui;
public:
    MDIAreaImplementation();
    ~MDIAreaImplementation();
    void setupUi(QWidget* parent);
     void showView(tlp::View* view);
     void drawAllViews();
     QList<tlp::View*> getAllViews()const{
         return QList<tlp::View*>();
     }
     void saveToProject(tlp::TulipProject*, QMap<tlp::Graph *, QString>, tlp::PluginProgress*){

     }

     void restoreFromProject(tlp::TulipProject*, QMap<QString, tlp::Graph *> , tlp::PluginProgress*){

     }
     virtual void closeView(tlp::View*){}
     virtual void closeAll(){}
};


class WorkspacePanelImplementation: public ViewManagerImplementation{

    Ui::WorkspacePanelImplementation* _ui;
    tlp::GraphHierarchiesModel* _model;
public:
    WorkspacePanelImplementation();
    ~WorkspacePanelImplementation();
    void setupUi(QWidget* parent);
     void showView(tlp::View *view);
     void drawAllViews();
     QList<tlp::View*> getAllViews()const;
     void saveToProject(tlp::TulipProject* project, QMap<tlp::Graph *, QString> rootIds, tlp::PluginProgress*);
     void restoreFromProject(tlp::TulipProject* project, QMap<QString, tlp::Graph *> rootIds, tlp::PluginProgress*);
     void closeView(tlp::View*);
     virtual void closeAll();
};

#endif // VIEWMANAGER_H
