#include <log.h>

#include <tulip/TlpQtTools.h>

#include <QStatusBar>
#include <QApplication>
#include <QIcon>

using namespace std;
using namespace tlp;

LogStringBuf OutputStream(Log::INFO_LEVEL);
LogStringBuf errorStream(Log::WARN_LEVEL);

std::streambuf* coutStream=NULL;
std::streambuf* cerrStream=NULL;

std::ostream consoleOut(NULL);
std::ostream consoleErrOut(NULL);

LogManager* LogManager::instance = NULL;

LogManager::LogManager()
{
}

LogManager::LogManager(const LogManager&):Log(){
}
LogManager* LogManager::getInstance(){
    if(instance == NULL){
        instance = new LogManager();
    }
    return instance;
}

LogManager::~LogManager(){
    qDeleteAll(logList);
}

bool LogManager::writeInLogs(Level level,const std::string& msg){    
    return getInstance()->writeLog(level,msg);
}

bool LogManager::writeLog(Level level,const string& msg){
    foreach(Log *l, logList) {
        l->writeLog(level,msg);
    }
    return true;
}

void LogManager::addLogToList(Log* log){
    logList << log;
}

void LogManager::removeLogFromList(Log* log){
    int i = logList.indexOf(log);
    if(i!=-1){
        logList.removeOne(logList[i]);
    }
}


STDLog::STDLog(ostream& out,Level levelFilter):out(out),_levelFilter(levelFilter){

}
STDLog::~STDLog(){

}
bool STDLog::writeLog(Level level,const std::string& msg){
    if(level == _levelFilter){
        out<<msg<<std::endl;
        return out.fail();
    }else{
        return true;
    }
}
LogStream logstream(){
    return LogStream(Log::DEBUG_LEVEL);
}

LogModel::LogModel(QObject* parent):QAbstractTableModel(parent){
}

int LogModel::rowCount(const QModelIndex &) const{
    return _messages.size();
}
QVariant LogModel::data(const QModelIndex &index, int role) const{

    if(!hasIndex(index.row(),index.column(),index.parent())){
        return QVariant();
    }

    if(role == Qt::FontRole){
        QFont font;
        if(!_messages[index.row()].seen){
            font.setBold(true);
        }
        return font;
    }

    if(index.column()==0){//Level
        if(role == Qt::EditRole ){
            return QVariant::fromValue<Level>(_messages[index.row()].level);
        }else if(role == Qt::DecorationRole){
            return icon(_messages[index.row()].level);
        }
    }else if(index.column()==1){//Time
        if(role == Qt::DisplayRole){
            return _messages[index.row()].time.toString("hh:mm:ss.zzz");
        }
    }else if(index.column() == 2){//Message
        if(role == Qt::DisplayRole){
            return _messages[index.row()].message;
        }else if(role == Qt::ForegroundRole){
            return color(_messages[index.row()].level);
        }
    }

    if(role == LevelRole){
        return QVariant::fromValue<Log::Level>(_messages[index.row()].level);
    }
    return QVariant();
}

QVariant LogModel::headerData(int section, Qt::Orientation orientation, int role) const{
    if(orientation == Qt::Horizontal){
        if(section==0){//Level
            if(role == Qt::DisplayRole ){
                return tr("Level");
            }
        }else if(section==1){//Time
            if(role == Qt::DisplayRole ){
                return tr("Time");
            }
        }else if(section == 2){//Message
            if(role == Qt::DisplayRole ){
                return tr("Message");
            }
        }
    }
    return QVariant();
}


QIcon LogModel::icon(Level level)const{
    switch(level){
    case FATAL_LEVEL:
    case ERROR_LEVEL:
        return QIcon(":/logwidget/icons/16/dialog-error.png");
        break;
    case WARN_LEVEL:
        return QIcon(":/logwidget/icons/16/dialog-warning.png");
        break;
    case INFO_LEVEL:
    case TRACE_LEVEL:
        return QIcon(":/logwidget/icons/16/dialog-information.png");
        break;
    case DEBUG_LEVEL:
        break;
    }
    return QIcon();
}
QColor LogModel::color(Level level)const{
    switch(level){
    case FATAL_LEVEL:
    case ERROR_LEVEL:
        return Qt::red;
        break;
    case WARN_LEVEL:
        return Qt::darkYellow;
        break;
    case INFO_LEVEL:
    case TRACE_LEVEL:
        return Qt::black;
        break;
    case DEBUG_LEVEL:
        return Qt::blue;
        break;
    }
    return Qt::black;
}



void LogModel::sort(int column, Qt::SortOrder order){
    emit layoutAboutToBeChanged();
    if(column == 0){
        qStableSort(_messages.begin(),_messages.end(),LevelMessageSorter(order));
    }else if(column == 1){
        qStableSort(_messages.begin(),_messages.end(),TimeMessageSorter(order));
    }
    emit layoutChanged();
}

int LogModel::unseenMessageCount()const{
    int count = 0;
    foreach(const LogMessage m, _messages) {
        if(!m.seen){
            ++count;
        }
    }
    return count;
}

void LogModel::markAllSeen(){
    beginResetModel();
    foreach(LogMessage m, _messages) {
        if(!m.seen){
            m.seen=true;
        }
    }
    endResetModel();
}


void redirectConsoleStreams(){

    coutStream = std::cout.rdbuf();
    std::cout.rdbuf(&OutputStream);

    cerrStream = std::cerr.rdbuf();
    std::cerr.rdbuf(&errorStream);

#ifndef NDEBUG
    consoleOut.rdbuf(coutStream);
    consoleErrOut.rdbuf(cerrStream);
    LogManager::getInstance()->addLogToList(new STDLog(consoleOut,Log::INFO_LEVEL));
    LogManager::getInstance()->addLogToList(new STDLog(consoleErrOut,Log::ERROR_LEVEL));
#endif

}

void restoreConsoleStreams(){
    std::cout.rdbuf(coutStream);
    std::cerr.rdbuf(cerrStream);
}

