#include "logwidget.h"
#include "ui_logwidget.h"

#include <log.h>

//Register resources
struct QtLogWidgetResourceInitializer
{
    QtLogWidgetResourceInitializer()
    {
        Q_INIT_RESOURCE(logwidget);
    }
};

extern "C"{
QtLogWidgetResourceInitializer logWidgetStaticLibInitializer;
}


LogModelSortFilterProxy::LogModelSortFilterProxy(QObject* parent):QSortFilterProxyModel(parent),_levelsFlags(LogLevels(Log::INFO_LEVEL|Log::WARN_LEVEL|Log::ERROR_LEVEL|Log::FATAL_LEVEL)){
}


bool LogModelSortFilterProxy::filterAcceptsRow ( int source_row, const QModelIndex&) const{
    Log::Level l = sourceModel()->index(source_row,0).data(LogModel::LevelRole).value<Log::Level>();
    return _levelsFlags.testFlag(l);
}
bool LogModelSortFilterProxy::lessThan ( const QModelIndex & left, const QModelIndex & right ) const{
    return left.data(LogModel::LevelRole).value<Log::Level>() < right.data(LogModel::LevelRole).value<Log::Level>();
}

LogWidget::LogWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::LogWidget),_model(NULL),_filter(NULL)
{
    ui->setupUi(this);
    _filter = new LogModelSortFilterProxy(ui->tableView);
    ui->tableView->setModel(_filter);
    ui->informationButton->setDefaultAction(new QAction(QIcon(QPixmap(":/logwidget/icons/16/dialog-information.png")),tr("Information"),ui->informationButton));
    ui->informationButton->defaultAction()->setCheckable(true);
    ui->informationButton->defaultAction()->setChecked(true);
    ui->informationButton->defaultAction()->setData(QVariant::fromValue<Log::Level>(Log::INFO_LEVEL));

    ui->warningButton->setDefaultAction(new QAction(QIcon(QPixmap(":/logwidget/icons/16/dialog-warning.png")),tr("Warning"),ui->warningButton));
    ui->warningButton->defaultAction()->setCheckable(true);
    ui->warningButton->defaultAction()->setChecked(true);
    ui->warningButton->defaultAction()->setData(QVariant::fromValue<Log::Level>(Log::WARN_LEVEL));

    ui->errorButton->setDefaultAction(new QAction(QIcon(QPixmap(":/logwidget/icons/16/dialog-error.png")),tr("Error"),ui->errorButton));
    ui->errorButton->defaultAction()->setCheckable(true);
    ui->errorButton->defaultAction()->setChecked(true);
    ui->errorButton->defaultAction()->setData(QVariant::fromValue<Log::Level>(Log::ERROR_LEVEL));

    connect(ui->informationButton->defaultAction(),SIGNAL(toggled(bool)),SLOT(filterToggled(bool)));
    connect(ui->warningButton->defaultAction(),SIGNAL(toggled(bool)),SLOT(filterToggled(bool)));
    connect(ui->errorButton->defaultAction(),SIGNAL(toggled(bool)),SLOT(filterToggled(bool)));

}

LogWidget::~LogWidget()
{
    delete ui;
}

void LogWidget::setLogModel(LogModel* model){
    _model = model;
    _filter->setSourceModel(model);
}

LogModel* LogWidget::logModel()const{
    return static_cast<LogModel*>(_filter->sourceModel());
}

void LogWidget::clearList(){
    _model->clearMessages();
}

void LogWidget::markSeen(){
    _model->markAllSeen();
}

void LogWidget::setVisible(bool visible){
    //If the widget was visible so clear unseen message
    if(isVisible()){
        _model->markAllSeen();
    }
    QWidget::setVisible(visible);
}

void LogWidget::filterToggled(bool enabled){
    QAction* action = dynamic_cast<QAction*>(sender());
    Log::Level f = action->data().value<Log::Level>();
    LogLevels filter = _filter->filterFlags();
    if(enabled){
        filter |= f;
    }else{
        filter &= ~f;
    }
    _filter->setFilterFlag(filter);
}
