#include "abstractportgraphview.h"

#include <tulip/GlMainWidget.h>
#include <tulip/GlGraphComposite.h>
#include <tulip/TlpQtTools.h>

#include <portgraph/PortGraphModel.h>

#include <porgy/pluginparametersconfigurationdialog.h>

#include <QGraphicsView>
#include <QMessageBox>

using namespace tlp;
using namespace std;

AbstractPortGraphView::AbstractPortGraphView()
{
}
AbstractPortGraphView::~AbstractPortGraphView(){
}


void AbstractPortGraphView::setupWidget(){
    AbstractPorgyGraphView::setupWidget();
    setOverviewVisible(false);
}

void AbstractPortGraphView::redrawGraph(){
    Graph *g = graph();
    PortGraph* pg = buildPortGraph(g);
    std::list<std::string> plugins = pg->layoutPluginsList();
    if(plugins.empty()){
        std::cerr<<"No layout plugin for the graph "<<graph()->getName()<<std::endl;
        return;
    }else {
        std::string pluginName;
        tlp::DataSet parameters;
        bool ok=false;
        if(plugins.size() == 1){
            pluginName = plugins.front();
            parameters = PluginParametersConfigurationDialog::getParameters(graphicsView(),"Configure draw plugin",tlp::PluginLister::instance()->getPluginParameters(pluginName),ok, NULL,g);
        }else{
            assert(false);
        }

        Observable::holdObservers();
        if(!pluginName.empty() && ok){//If the plugin is valid and the user doesn't click on cancel button.
            string errMsg;
            graph()->getRoot()->push(false);
            if(pg->redraw(pluginName,parameters,errMsg)){
                Observable::unholdObservers();
                centerWithAnimation();
            }else{
                QMessageBox::critical(this->getGlMainWidget()->parentWidget(), "Cannot redraw graph", trUtf8("Error when drawing graph ")+tlp::tlpStringToQString(graph()->getName())+" : "+tlp::tlpStringToQString(errMsg));
                graph()->getRoot()->pop(false);
                Observable::unholdObservers();
            }
        }

    }

    delete pg;
}

void AbstractPortGraphView::invertSelection(SelectedEntity &selectedentity){
    Graph *g = graph();
    PortGraph* pg = buildPortGraph(g);
    BooleanProperty *elementSelected = pg->getSelectionProperty();
    // selection add/remove graph item
    node n(selectedentity.getNode());
    if (n.isValid()){
        pg->getPortNode(n)->select(!elementSelected->getNodeValue(n));
    }
    else{
        edge e(selectedentity.getEdge());
        elementSelected->setEdgeValue(e, !elementSelected->getEdgeValue(e));
    }
    delete pg;
}


void AbstractPortGraphView::setNodeLabelVisibility(bool state) {
    getGlMainWidget()->getScene()->getGlGraphComposite()->getRenderingParametersPointer()->setViewNodeLabel(state);
    emit drawNeeded();
}

void AbstractPortGraphView::setNodeLabelScaled(bool state) {
    getGlMainWidget()->getScene()->getGlGraphComposite()->getRenderingParametersPointer()->setLabelScaled(state);
    emit drawNeeded();
}

void AbstractPortGraphView::showArrows(bool state) {
    getGlMainWidget()->getScene()->getGlGraphComposite()->getRenderingParametersPointer()->setViewArrow(state);
    emit drawNeeded();
}
