/**
 *
 * This file is part of Tulip (www.tulip-software.org)
 *
 * Authors: David Auber and the Tulip development Team
 * from LaBRI, University of Bordeaux 1 and Inria Bordeaux - Sud Ouest
 *
 * Tulip is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Tulip is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 */
#include <tulip/Node.h>
#include <tulip/ForEach.h>
#include <tulip/Graph.h>
#include <tulip/ColorProperty.h>
using namespace tlp;
using namespace std;

template<typename PropertyType,typename NodeType,typename EdgeType>
void AbstractGraphMorphing::computeMiddleValueForProperty(tlp::Graph* fromGraph,PropertyType* from,tlp::Graph* toGraph,PropertyType* to,PropertyType* outFrom,PropertyType* outTo)const{
    node n;
    forEach(n,fromGraph->getNodes()){
        if(toGraph->isElement(n)){
            NodeType r = computeMiddleValue<NodeType>(from->getNodeValue(n),to->getNodeValue(n));
            outFrom->setNodeValue(n,r);
            outTo->setNodeValue(n,r);
        }
    }
    edge e;
    forEach(e,fromGraph->getEdges()){
        if(toGraph->isElement(e)){
            EdgeType r = computeMiddleValue<EdgeType>(from->getEdgeValue(e),to->getEdgeValue(e));
            outFrom->setEdgeValue(e,r);
            outTo->setEdgeValue(e,r);
        }
    }
}


template<typename T>
T AbstractGraphMorphing::computeMiddleValue(const T& from,const T& to)const{
   return from + ((to - from) / 2.f);
}

