#ifndef GLITEM_H
#define GLITEM_H

#include <QDeclarativeItem>

#include <tulip/GlScene.h>

class GlItem : public QDeclarativeItem
{
    Q_OBJECT

    tlp::GlScene _scene;
    QSizeF _size;
    bool _redrawNeeded;
    unsigned char *_renderingStore;

public:
    GlItem(QDeclarativeItem* parent=NULL);
    ~GlItem();
    const tlp::GlScene& scene()const{
        return _scene;
    }
    tlp::GlScene& scene(){
        return _scene;
    }
    void paint(QPainter *, const QStyleOptionGraphicsItem *, QWidget *);

    /** @brief Select nodes and edges in a region of the screen
     *
     *  Select all nodes and edges lying in the area of the screen of given width and height,
     *  and with its upper-left corner at (x,y)
     *  @param selectedNode filled by the method with the nodes found in the region
     *  @param selectedEdge filled by the method with the edges found in the region
     */
    void pickNodesEdges(const int x, const int y,
                        const int width, const int height,
                        std::vector<tlp::SelectedEntity> &selectedNode, std::vector<tlp::SelectedEntity> &seletedEdge,
                        tlp::GlLayer* layer=NULL);
    /** @brief Select a node or edge at a point
     *  Select either a node or edge at point (x,y)
     *  @param type tells what has been found: NODE, EDGE
     *  @return true if something has been found, false otherwise
     */
    bool pickNodesEdges(const int x, const int y,
                        tlp::SelectedEntity &selectedEntity,
                        tlp::GlLayer* layer=NULL);

    /**
     * @brief Function to do picking on entities.
     * It just calls selectEntities on the GlScene instance.
     * @param x screen coordinates
     * @param y screen coordinates
     * @param width screen size
     * @param height screen size
     * @param pickedEntities filled with entity under the selection screen rectangle
     * @param layer if you want to do the selection only on one GlLayer
     */
    bool pickGlEntities(const int x, const int y,
                        const int width, const int height,
                        std::vector<tlp::SelectedEntity>
                        &pickedEntities,
                        tlp::GlLayer* layer=NULL);
    /**
     * @brief Function to do picking on entities.
     * It just calls selectEntities on the GlScene instance with a small window of twelve pixels.
     * @param x screen coordinates
     * @param y screen coordinates
     * @param pickedEntities filled with entity under the selection screen rectangle
     * @param layer if you want to do the selection only on one GlLayer
     */
    bool pickGlEntities(const int x, const int y,
                        std::vector<tlp::SelectedEntity>
                        &pickedEntities,
                        tlp::GlLayer* layer=NULL);

    void draw(){
        _redrawNeeded = true;
    }

    QPixmap createPixmap(const QSize& size);

protected:
    void geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry);    

private:

    void makeCurrent();
    void updateGlWidgetGeometry(const QRectF &newGeometry);
};

#endif // GLITEM_H
