/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "porgysmallmultiplestraceview.h"
#include "porgysmallmultiplesconfigurationwidget.h"
#include "animationconfigurationwidget.h"
#include "smallmultiplesconfigurationwidget.h"
#include "titlemorphing.h"
#include "GlSceneMorphing.h"
#include "VisualPropertiesMorphing.h"
#include "SimpleCameraMorphing.h"
#include "glitem.h"

#include <tulip/GlMainWidget.h>
#include <tulip/GlOffscreenRenderer.h>
#include <tulip/TlpQtTools.h>
#include <tulip/DrawingTools.h>
#include <tulip/GlGraphComposite.h>
#include <tulip/GlVertexArrayManager.h>
#include <tulip/GlMetaNodeRenderer.h>
#include <tulip/GlTextureManager.h>
#include <tulip/TlpTools.h>

#include <QGLFormat>
#include <QGraphicsObject>
#include <QResizeEvent>
#include <QPropertyAnimation>
#include <QParallelAnimationGroup>
#include <QDeclarativeContext>
#include <QFile>

using namespace tlp;
using namespace std;


static QGLFormat GlFormatInitialization() {
    QGLFormat tmpFormat;
    tmpFormat.setDirectRendering(true);
    tmpFormat.setDoubleBuffer(true);
    tmpFormat.setAccum(false);
    tmpFormat.setStencil(true);
    tmpFormat.setOverlay(false);
    tmpFormat.setDepth(true);
    tmpFormat.setRgba(true);
    tmpFormat.setAlpha(true);
    tmpFormat.setOverlay(false);
    tmpFormat.setStereo(false);
    tmpFormat.setSampleBuffers(true);
    return tmpFormat;
}


DeclarativeGraphicsView::DeclarativeGraphicsView(QWidget* parent):QDeclarativeView(parent),_animationItem(NULL),_smallMultipleItem(NULL),_rulePreviewItem(NULL),_stateName(NULL){
}

void DeclarativeGraphicsView::resizeEvent(QResizeEvent *event){
    QGraphicsScene *mascene = scene();
    if (mascene!=NULL) {
        mascene->setSceneRect(QRect(QPoint(0, 0), size()));
    }
    QDeclarativeView::resizeEvent(event);
}

GlItem* DeclarativeGraphicsView::rulePreviewItem(){
    if(_rulePreviewItem == NULL){
        assert(rootObject() != NULL);
        _rulePreviewItem = dynamic_cast<GlItem*>(rootObject()->findChild<GlItem*>("RulePreview"));
        assert(_rulePreviewItem != NULL);
    }
    return _rulePreviewItem;
}

GlItem* DeclarativeGraphicsView::animationItem(){
    if(_animationItem == NULL){
        assert(rootObject() != NULL);
        _animationItem = dynamic_cast<GlItem*>(rootObject()->findChild<QObject*>("GlItem"));
        assert(_animationItem != NULL);
    }
    return _animationItem;
}

GlItem* DeclarativeGraphicsView::smallMultipleItem(){
    if(_smallMultipleItem == NULL){
        assert(rootObject() != NULL);
        _smallMultipleItem = dynamic_cast<GlItem*>(rootObject()->findChild<QObject*>("SmallMultipleItem"));
        assert(_smallMultipleItem != NULL);
    }
    return _smallMultipleItem;
}


QObject* DeclarativeGraphicsView::stateName(){
    if(_stateName == NULL){
        assert(rootObject() != NULL);
        _stateName = rootObject()->findChild<QObject*>("StateName");
        assert(_stateName != NULL);
    }
    return _stateName;
}
void DeclarativeGraphicsView::setStateName(const QString& name){
    stateName()->setProperty("text",name);
}
void DeclarativeGraphicsView::setRulePreviewName(const QString& name){
    rootObject()->setProperty("rulePreviewLabel",name);
}

void DeclarativeGraphicsView::setRemoteMode(RemoteMode mode){
    rootObject()->setProperty("remoteMode",mode==Playing?0:1);
}

void DeclarativeGraphicsView::setRulePreviewVisible(bool visible){
    rootObject()->setProperty("rulePreviewVisible",visible);
}
void DeclarativeGraphicsView::setAnimationVisible(bool visible){
    rootObject()->setProperty("animationVisible",visible);
}

void DeclarativeGraphicsView::setSmallMultiplesVisible(bool visible){
    rootObject()->setProperty("smallMultipleVisible",visible);
}

void DeclarativeGraphicsView::setRulePreviewGeometry(RulePreviewGeometry geometry){
    QString value;
    switch (geometry) {
    case Small_Rule_Preview:
        value = QString("Small");
        break;
    case Medium_Rule_Preview:
        value = QString("Medium");
        break;
    case Large_Rule_Preview:
        value = QString("Large");
        break;
    default:
        break;
    }
    if(!value.isEmpty()){
        rootObject()->setProperty("rulePreviewGeometry",value);
    }
}


PorgySmallMultiplesTraceView::PorgySmallMultiplesTraceView(const PluginContext *):_graphicsView(NULL),_configurationWidget(NULL),_animationConfigurationWidget(NULL),_smallMultiplesConfigurationWidget(NULL),_currentState(0),_currentAnimation(NULL),_goNextStep(false),_smallMultipleGraph(NULL){
}

PorgySmallMultiplesTraceView::~PorgySmallMultiplesTraceView(){
    clearRedrawTriggers();
    clearTextures();
    if(_currentAnimation != NULL){
        _currentAnimation->disconnect(this);
        _currentAnimation->stop();
        _currentAnimation->deleteLater();
        _visualProperties.deleteProperties();
    }
    delete _configurationWidget;
    delete _animationConfigurationWidget;
    delete _smallMultiplesConfigurationWidget;
    delete _graphicsView;
    delete _smallMultipleGraph;
}

QGraphicsView* PorgySmallMultiplesTraceView::graphicsView() const{
    return _graphicsView;
}



void PorgySmallMultiplesTraceView::setAnimationData(tlp::Graph* graph,GraphVisualProperties& properties){
    removeRedrawTriggers(_graphicsView->animationItem()->scene());
    loadGraphInScene(_graphicsView->animationItem()->scene(),graph);
    properties.copyToInputData(_graphicsView->animationItem()->scene().getGlGraphComposite()->getInputData());
    addRedrawTriggers(_graphicsView->animationItem()->scene());
}

void PorgySmallMultiplesTraceView::loadGraphInScene(tlp::GlScene& scene,tlp::Graph* graph){

    GlLayer *layer = scene.getLayer("Main");

    if (layer == NULL) {
        layer = new GlLayer("Main");
        scene.addExistingLayer(layer);
    }
    GlGraphComposite* oldComposite = dynamic_cast<GlGraphComposite*>(layer->findGlEntity("graph"));

    GlGraphComposite* graphComposite=new GlGraphComposite(graph);
    if(oldComposite != NULL){
        GlGraphRenderingParameters param=oldComposite->getRenderingParameters();
        GlMetaNodeRenderer *metaNodeRenderer=oldComposite->getInputData()->getMetaNodeRenderer();
        oldComposite->getInputData()->setMetaNodeRenderer(NULL,false);
        graphComposite->setRenderingParameters(param);
        metaNodeRenderer->setInputData(graphComposite->getInputData());
        graphComposite->getInputData()->setMetaNodeRenderer(metaNodeRenderer);

        if(oldComposite->getInputData()->graph==graph) {
            oldComposite->getInputData()->deleteGlVertexArrayManagerInDestructor(false);
            delete graphComposite->getInputData()->getGlVertexArrayManager();
            graphComposite->getInputData()->setGlVertexArrayManager(oldComposite->getInputData()->getGlVertexArrayManager());
            graphComposite->getInputData()->getGlVertexArrayManager()->setInputData(graphComposite->getInputData());
        }
    }
    layer->addGlEntity(graphComposite,"graph");
    graphComposite->getRenderingParametersPointer()->setEdgeColorInterpolate(false);
    delete oldComposite;
}

void PorgySmallMultiplesTraceView::removeRedrawTriggers(tlp::GlScene& scene){    
    GlGraphComposite* composite = scene.getGlGraphComposite();
    removeRedrawTrigger(composite->getGraph());
    std::set<tlp::PropertyInterface*> properties = composite->getInputData()->properties();
    for(std::set<tlp::PropertyInterface*>::iterator it = properties.begin() ; it != properties.end() ; ++it){
        removeRedrawTrigger(*it);
    }
}

void PorgySmallMultiplesTraceView::addRedrawTriggers(tlp::GlScene& scene){    
    GlGraphComposite* composite = scene.getGlGraphComposite();
    addRedrawTrigger(composite->getGraph());
    std::set<tlp::PropertyInterface*> properties = composite->getInputData()->properties();
    for(std::set<tlp::PropertyInterface*>::iterator it = properties.begin() ; it != properties.end() ; ++it){
        addRedrawTrigger(*it);
    }
}

void PorgySmallMultiplesTraceView::setupUi(){
    _graphicsView = new DeclarativeGraphicsView();
    _graphicsView->setRenderHints(QPainter::SmoothPixmapTransform | QPainter::Antialiasing | QPainter::TextAntialiasing);
    _graphicsView->setViewport(new QGLWidget(GlFormatInitialization(), 0, GlMainWidget::getFirstQGLWidget()));
    _graphicsView->installEventFilter(this);
    qmlRegisterType<GlItem>("Tulip", 1, 0, "GlItem");

    _graphicsView->rootContext()->setContextProperty("stateModel", &_states);//Give the state model to the list.
    _graphicsView->setSource(QUrl("qrc:/traceAnimationView/qml/traceAnimationView.qml"));

    _graphicsView->setResizeMode(QDeclarativeView::SizeRootObjectToView);

    _smallMultipleGraph = tlp::newGraph();
    Observable::holdObservers();
    loadGraphInScene(_graphicsView->smallMultipleItem()->scene(),_smallMultipleGraph);
    GlGraphInputData *inputData = _graphicsView->smallMultipleItem()->scene().getGlGraphComposite()->getInputData();
    inputData->getElementColor()->setAllNodeValue(_graphicsView->smallMultipleItem()->scene().getBackgroundColor());
    inputData->getElementShape()->setAllNodeValue(4);
    inputData->getElementLabelPosition()->setAllNodeValue(2);
    inputData->getElementFontSize()->setAllNodeValue(2);
    inputData->getElementFont()->setAllNodeValue(TulipBitmapDir + "font.ttf");
    inputData->getElementFont()->setAllEdgeValue(TulipBitmapDir + "font.ttf");
    inputData->getElementBorderWidth()->setAllNodeValue(1);
    _graphicsView->smallMultipleItem()->scene().getGlGraphComposite()->getRenderingParametersPointer()->setFontsType(1);
    _graphicsView->smallMultipleItem()->scene().getGlGraphComposite()->getRenderingParametersPointer()->setLabelScaled(false);
    _graphicsView->smallMultipleItem()->scene().getGlGraphComposite()->getRenderingParametersPointer()->setEdgeColorInterpolate(false);
    Observable::unholdObservers();

    //Build configuration widget.
    _configurationWidget = new PorgySmallMultiplesConfigurationWidget();
    connect(_configurationWidget,SIGNAL(intermediateStatesChanged()),this,SLOT(updateIntermediateStates()));
    _animationConfigurationWidget = new AnimationConfigurationWidget();
    connect(_animationConfigurationWidget,SIGNAL(rulePreviewGeometryChanged()),SLOT(rulePreviewGeometryChanged()));
    _smallMultiplesConfigurationWidget = new SmallMultiplesConfigurationWidget();
    connect(_smallMultiplesConfigurationWidget,SIGNAL(smallMultiplesLayoutUpdated()),this,SLOT(updateSmallMultipleLayout()));
    connect(_smallMultiplesConfigurationWidget,SIGNAL(scaleTitles(bool)),this,SLOT(scaleSmallMultiplesTitles(bool)));
    connect(_smallMultiplesConfigurationWidget, SIGNAL(spaceChanged(double)), this, SLOT(spacechanged(double)));

    connect(_graphicsView->rootObject(),SIGNAL(play()),this,SLOT(start()));
    connect(_graphicsView->rootObject(),SIGNAL(pause()),this,SLOT(pause()));
    connect(_graphicsView->rootObject(),SIGNAL(stop()),this,SLOT(stop()));
    connect(_graphicsView->rootObject(),SIGNAL(seekBackward()),this,SLOT(seekBackward()));
    connect(_graphicsView->rootObject(),SIGNAL(seekForward()),this,SLOT(seekForward()));
    connect(_graphicsView->rootObject(),SIGNAL(skipBackward()),this,SLOT(seekFirst()));
    connect(_graphicsView->rootObject(),SIGNAL(skipForward()),this,SLOT(seekLast()));


    //connect display/hide widget
    connect(_configurationWidget,SIGNAL(displaySmallMultiplesView(bool)),this,SLOT(displaySmallMultipleWidget(bool)));
    connect(_configurationWidget,SIGNAL(displayAnimationView(bool)),this,SLOT(displayAnimationWidget(bool)));
    connect(_configurationWidget,SIGNAL(scalelabels(bool)),this,SLOT(scalelabels(bool)));
    connect(_configurationWidget,SIGNAL(nodeLabelsVisibilityChanged(bool)),this,SLOT(updateNodeLabesVisibility(bool)));

}

QGraphicsItem* PorgySmallMultiplesTraceView::centralItem()const{
    return _graphicsView->rootObject();
}

bool PorgySmallMultiplesTraceView::eventFilter(QObject* obj, QEvent* event) {
  if (event->type() == QEvent::Resize) {
    // ensure automatic resize of the viewport
    QResizeEvent *resizeEvent = static_cast<QResizeEvent *>(event);
    graphicsView()->viewport()->setFixedSize(resizeEvent->size());
    // same for the configuration widgets
    QList<QWidget *> list = configurationWidgets();

    if(!list.isEmpty()) {   //test if the current view has a configuration widget
      QWidget *pqw = list.first()->parentWidget()->parentWidget();
      QSize sSize = pqw->size();
      sSize.setHeight(resizeEvent->size().height() - 50);
      pqw->resize(sSize);
      sSize.setHeight(resizeEvent->size().height() - 60);
      sSize = list.first()->size();
      foreach(QWidget *c, list) {   //resize each configuration widget
        c->resize(sSize);
      }
    }

    return true;
  }

  // standard event processing
  return View::eventFilter(obj, event);
}

QString PorgySmallMultiplesTraceView::configurationWidgetsStyleSheet() const {
    QFile css(":/traceAnimationView/css/view_configurationtab.css");
    css.open(QIODevice::ReadOnly);
    QString style(css.readAll());
    css.close();
    return style;
}

QList<QWidget*> PorgySmallMultiplesTraceView::configurationWidgets()const{
    return QList<QWidget*>()<<_configurationWidget<<_animationConfigurationWidget<<_smallMultiplesConfigurationWidget;
}

void PorgySmallMultiplesTraceView::graphChanged(tlp::Graph* graph){    
    clearRedrawTriggers();

    _states.setData(graph);
    _states.updateStateList(_configurationWidget->intermediateStates());
    _visualProperties.deleteProperties();    //Clear old properties.

    //Invalid state display empty graph
    if(_states.stateNumber()!=0){
        _states.updateStateList(_configurationWidget->intermediateStates());
        updateSmallMultiples(Reset);
        //Create visual properties for the graph
        _visualProperties.setElementColor( new ColorProperty(graph->getRoot()));
        _visualProperties.setElementBorderColor( new ColorProperty(graph->getRoot()));
        _visualProperties.setElementLabelColor( new ColorProperty(graph->getRoot()));
        _visualProperties.setElementLayout(new LayoutProperty(graph->getRoot()));
        _visualProperties.setElementBorderWidth(new DoubleProperty(graph->getRoot()));

        //Update animation
        _currentState = 0;
        _graphicsView->setStateName(QString::fromUtf8(_states.getState(0).name.c_str()));
        GraphState &state =_states.getState(0);
        //Overload visual properties.
        node n;
        forEach(n,state.graph->getNodes()){
            if(state.properties.getElementColor()!=NULL){
                _visualProperties.getElementColor()->setNodeValue(n,state.properties.getElementColor()->getNodeValue(n));
            }
            if(state.properties.getElementBorderColor()!=NULL){
                _visualProperties.getElementBorderColor()->setNodeValue(n,state.properties.getElementBorderColor()->getNodeValue(n));
            }
            if(state.properties.getElementLabelColor()!=NULL){
                _visualProperties.getElementLabelColor()->setNodeValue(n,state.properties.getElementLabelColor()->getNodeValue(n));
            }
            if(state.properties.getElementLayout()!=NULL){
                _visualProperties.getElementLayout()->setNodeValue(n,state.properties.getElementLayout()->getNodeValue(n));
            }
            if(state.properties.getElementBorderWidth()!=NULL){
                _visualProperties.getElementBorderWidth()->setNodeValue(n,state.properties.getElementBorderWidth()->getNodeValue(n));
            }

        }

        edge e;
        forEach(e,state.graph->getEdges()){
            if(state.properties.getElementColor()!=NULL){
                _visualProperties.getElementColor()->setEdgeValue(e,state.properties.getElementColor()->getEdgeValue(e));
            }
            if(state.properties.getElementBorderColor()!=NULL){
                _visualProperties.getElementBorderColor()->setEdgeValue(e,state.properties.getElementBorderColor()->getEdgeValue(e));
            }
            if(state.properties.getElementLabelColor()!=NULL){
                _visualProperties.getElementLabelColor()->setEdgeValue(e,state.properties.getElementLabelColor()->getEdgeValue(e));
            }
            if(state.properties.getElementLayout()!=NULL){
                _visualProperties.getElementLayout()->setEdgeValue(e,state.properties.getElementLayout()->getEdgeValue(e));
            }
            if(state.properties.getElementBorderWidth()!=NULL){
                _visualProperties.getElementBorderWidth()->setEdgeValue(e,state.properties.getElementBorderWidth()->getEdgeValue(e));
            }
        }

        loadGraphInScene(_graphicsView->animationItem()->scene(),_states.getState(0).graph);
        _visualProperties.copyToInputData(_graphicsView->animationItem()->scene().getGlGraphComposite()->getInputData());
        _graphicsView->animationItem()->scene().centerScene();
        _graphicsView->setEnabled(true);
    }else{
        _graphicsView->setEnabled(false);
    }

    addRedrawTriggers(_graphicsView->smallMultipleItem()->scene());//Listen small multiples
    addRedrawTriggers(_graphicsView->animationItem()->scene());//Listen animation

}

tlp::GlGraphRenderingParameters PorgySmallMultiplesTraceView::renderingParameters()const{
    GlGraphRenderingParameters parameters;
    parameters.setEdgeColorInterpolate(false);
    parameters.setLabelsDensity(0);
    parameters.setViewNodeLabel(_configurationWidget->nodeLabelsVisible());
    parameters.setLabelScaled(_configurationWidget->labelScaled());
    return parameters;
}

void PorgySmallMultiplesTraceView::spacechanged(double) {
    updateSmallMultiples(Update_Layout);
}

void PorgySmallMultiplesTraceView::scalelabels(bool scaled) {
    _graphicsView->animationItem()->scene().getGlGraphComposite()->getRenderingParametersPointer()->setLabelScaled(scaled);
    if(_graphicsView->rulePreviewItem()->scene().getGlGraphComposite()!=NULL){
        _graphicsView->rulePreviewItem()->scene().getGlGraphComposite()->getRenderingParametersPointer()->setLabelScaled(scaled);
    }
    emit drawNeeded();
}

void PorgySmallMultiplesTraceView::updateNodeLabesVisibility(bool visible){
    _graphicsView->animationItem()->scene().getGlGraphComposite()->getRenderingParametersPointer()->setViewNodeLabel(visible);
    if(_graphicsView->rulePreviewItem()->scene().getGlGraphComposite()!=NULL){
        _graphicsView->rulePreviewItem()->scene().getGlGraphComposite()->getRenderingParametersPointer()->setViewNodeLabel(visible);
    }
    emit drawNeeded();
}

void PorgySmallMultiplesTraceView::scaleSmallMultiplesTitles(bool scaled){
    _graphicsView->smallMultipleItem()->scene().getGlGraphComposite()->getRenderingParametersPointer()->setLabelScaled(scaled);
}

void PorgySmallMultiplesTraceView::updateSmallMultiples(SmallMultipleUpdateTypeFlags updateType){
    if(updateType.testFlag(Reset)){//Reset all elements
        clearTextures();
        int diff = _states.stateNumber()-_smallMultipleGraph->numberOfNodes();
        while(diff != 0){
            if(diff<0){
                _smallMultipleGraph->delNode(_smallMultipleGraph->getOneNode());
                ++diff;
            }else{
                _smallMultipleGraph->addNode();
                --diff;
            }
        }
        _textures.resize(_states.stateNumber());
    }
    int currentIndex=0;
    //Generate elements
    node n;
    forEach(n,_smallMultipleGraph->getNodes()){
        if(updateType.testFlag(Reset)){
            //Set element id
            _smallMultipleGraph->getProperty<IntegerProperty>("elementIndex")->setNodeValue(n,currentIndex);
        }
        if(updateType.testFlag(Update_Name)){
            //Element name
            _smallMultipleGraph->getProperty<StringProperty>("viewLabel")->setNodeValue(n,_states.getState(currentIndex).name);
        }
        if(updateType.testFlag(Update_texture)){
            //Element texture
            GlTextureManager::getInst().deleteTexture(_textures[currentIndex]);
            _smallMultipleGraph->getProperty<StringProperty>("viewTexture")->setNodeValue(n,generateTexture(currentIndex));
        }
        if(updateType.testFlag(Update_Layout)){
            //Element position
            int lineNumber = _smallMultiplesConfigurationWidget->smallMultiplesLayout() == ForceLineNumber?_smallMultiplesConfigurationWidget->lineNumber():ceil(_states.stateNumber()/round(sqrt(_states.stateNumber())));
            float elementsByRow = static_cast<float>(_states.stateNumber()) / static_cast<float>(lineNumber);
            //If the number of elements divided by the number of row is a whole number use it as number of element by row else use the ceil.
            elementsByRow = ceil(elementsByRow) == floor(elementsByRow)?elementsByRow:ceil(elementsByRow);
            int col = floor(static_cast<float>(currentIndex) / static_cast<float>(elementsByRow));
            int row = currentIndex % static_cast<int>(elementsByRow);
            // Compute spacing
            float s = 1.f + _smallMultiplesConfigurationWidget->spacing();
            _smallMultipleGraph->getProperty<LayoutProperty>("viewLayout")->setNodeValue(n,Coord(row * s, -col * s, 0));
        }
        ++currentIndex;
    }

    _graphicsView->smallMultipleItem()->scene().centerScene();


}

string PorgySmallMultiplesTraceView::generateTexture(unsigned int stateId){
    assert(stateId<_states.stateNumber());
    assert(stateId<_textures.size());
    GlTextureManager &textureManager = GlTextureManager::getInst();
    //Destruct old texture.
    string textureName = getTextureNameForId(stateId);
    textureManager.deleteTexture(textureName);
    //Create graph preview texture
    GlOffscreenRenderer* renderer = GlOffscreenRenderer::getInstance();
    renderer->setSceneBackgroundColor(Color(255,255,255));
    renderer->setViewPortSize(512,512);
    renderer->clearScene();
    GraphState &state = _states.getState(stateId);
    GlGraphComposite* composite = new GlGraphComposite(state.graph);
    composite->getRenderingParametersPointer()->setEdgeColorInterpolate(false);
    composite->getRenderingParametersPointer()->setViewNodeLabel(_configurationWidget->nodeLabelsVisible());
    composite->getRenderingParametersPointer()->setLabelScaled(_configurationWidget->labelScaled());
    //Overwrite default properties with new one.
    state.properties.copyToInputData(composite->getInputData());
    renderer->addGraphCompositeToScene(composite);
#ifdef __APPLE__
    renderer->renderScene(true,false);
#else
    renderer->renderScene(true,true);
#endif
    GLuint textureId = renderer->getGLTexture();
    //Register texture.
    textureManager.registerExternalTexture(textureName,textureId);
    _textures[stateId]=textureName;
    return textureName;
}

string PorgySmallMultiplesTraceView::getTextureNameForId(int stateId){
    return QStringToTlpString(QString("porgysmallmultiples")+QString::number(stateId));
}

void PorgySmallMultiplesTraceView::clearTextures(){
    GlTextureManager &textureManager = GlTextureManager::getInst();
    for(vector<string>::iterator it = _textures.begin(); it != _textures.end();++it){
        textureManager.deleteTexture(*it);
    }
}

void PorgySmallMultiplesTraceView::animate(unsigned int to){
    //Avoid video update during animation
    if(_configurationWidget->isEnabled()){
        _configurationWidget->setEnabled(false);
    }
    //There is already an animation stop it.
    if(_currentAnimation){
        //Avoid to receive the animation state changed signal
        _currentAnimation->disconnect(this);
        _currentAnimation->stop();
        _currentAnimation->deleteLater();
    }
    //tells to the control widget we are playing.
    _graphicsView->setRemoteMode(DeclarativeGraphicsView::Playing);//Set the pause icon on the play button
    _currentState=to;
    Observable::holdObservers();
    _currentAnimation=buildMorphingBetweenStates(to);
    Observable::unholdObservers();
    connect(_currentAnimation,SIGNAL(stateChanged(QAbstractAnimation::State,QAbstractAnimation::State)),SLOT(currentAnimationStateChanged(QAbstractAnimation::State,QAbstractAnimation::State)));
    _currentAnimation->start();
}

QAbstractAnimation* PorgySmallMultiplesTraceView::buildMorphingBetweenStates( unsigned int to){
    //Init state from current view
    GraphState fromState(_graphicsView->animationItem()->scene().getGlGraphComposite()->getGraph(),GraphVisualProperties(_graphicsView->animationItem()->scene().getGlGraphComposite()->getInputData(),VisualPropertiesFlags(AllColorProperties | ElementSize | ElementLayout | ElementBorderWidth)));
    GraphState &toState = _states.getState(to);
    //Compute frame number
    unsigned int framesCount = ((float)_animationConfigurationWidget->animationDuration()/1000.0)*_animationConfigurationWidget->framePerSecond();
    //Avoid invalid frame count.
    if(framesCount==0){
        framesCount = 1;
    }
    QPropertyAnimation *animation=NULL;
    //Use a simple graph morphing.
    if(fromState.graph == toState.graph){
        assert(fromState.properties.getElementColor()!=NULL);
        assert(toState.properties.getElementColor()!=NULL);
        animation =new VisualPropertiesMorphing(fromState.graph,fromState.properties,toState.properties,_visualProperties,framesCount,VisualPropertiesFlags(AllColorProperties | ElementLayout | ElementBorderWidth),this);
    }else{
        animation = new GlSceneMorphing(this,fromState.graph,fromState.properties,_visualProperties,toState.graph,toState.properties,_visualProperties,framesCount,&_graphicsView->animationItem()->scene(),VisualPropertiesFlags(AllColorProperties| ElementLayout | ElementBorderWidth),this);
    }
    animation->setDuration(_animationConfigurationWidget->animationDuration());

    //If the graph of the two different state is different update the rule widget.
    Graph* rule = toState.rule;
    bool rulePreviewIsVisible = _animationConfigurationWidget->rulePreviewGeometry() != No_Rule_Preview && rule != NULL;
    //Show the rule preview widget if it's marked as shown in the animation configuration widget and if the graph to display is not NULL.
    _graphicsView->setRulePreviewVisible(rulePreviewIsVisible);
    //Update the graph.
    if(rule != NULL && (_graphicsView->rulePreviewItem()->scene().getGlGraphComposite()== NULL || _graphicsView->rulePreviewItem()->scene().getGlGraphComposite()->getGraph() != rule)){

        loadGraphInScene(_graphicsView->rulePreviewItem()->scene(),rule);
        _graphicsView->rulePreviewItem()->scene().centerScene();
        _graphicsView->rulePreviewItem()->draw();
        _graphicsView->setRulePreviewName(QString::fromUtf8(rule->getName().c_str()));
    }

    QParallelAnimationGroup *graphAndCameraAnimation = new QParallelAnimationGroup(this);
    TitleMorphing* titleMorphing = new TitleMorphing(_graphicsView->stateName(),toState.name,framesCount,this);
    titleMorphing->setDuration(_animationConfigurationWidget->animationDuration());
    graphAndCameraAnimation->addAnimation(titleMorphing);
    graphAndCameraAnimation->addAnimation(animation);

    //Compute destination bounding box
    BoundingBox bboxTo = computeBoundingBox(toState.graph,toState.properties.getElementLayout()!=NULL?toState.properties.getElementLayout():toState.graph->getProperty<LayoutProperty>("viewLayout"),toState.properties.getElementSize()!=NULL?toState.properties.getElementSize():toState.graph->getProperty<SizeProperty>("viewSize"),toState.graph->getProperty<DoubleProperty>("viewRotation"),NULL);
    //Compute destination bounding box
    BoundingBox from = computeBoundingBox(fromState.graph,fromState.properties.getElementLayout(),fromState.properties.getElementSize(),fromState.graph->getProperty<DoubleProperty>("viewRotation"),NULL);
    if( !from.contains(bboxTo[0]) || !from.contains(bboxTo[1]) || bboxTo.width()*1.1 < from.width() || bboxTo.height()*1.1 < from.height()){
        //Add margin
        float widthMargin = bboxTo.width()*0.1;
        float heightMargin = bboxTo.height()*0.1;
        Vec3f newMin = bboxTo[0];
        newMin[0] -= widthMargin/2;
        newMin[1] -= heightMargin/2;
        Vec3f newMax = bboxTo[1];
        newMax[0] += widthMargin/2;
        newMax[1] += heightMargin/2;
        bboxTo.expand(newMin);
        bboxTo.expand(newMax);
        QPropertyAnimation* cameraAnimation = new SimpleCameraMorphing(_graphicsView->animationItem()->scene().getGraphCamera(),bboxTo,framesCount,this);
        cameraAnimation->setDuration(_animationConfigurationWidget->animationDuration());
        graphAndCameraAnimation->addAnimation(cameraAnimation);

    }

    return graphAndCameraAnimation;
}


void PorgySmallMultiplesTraceView::start(){    
    if(_currentAnimation && _currentAnimation->state()==QAbstractAnimation::Paused){
        _currentAnimation->resume();
    }else{
        _goNextStep = true;
        seekForward();
    }
}

void PorgySmallMultiplesTraceView::pause(){
    if(_currentAnimation && _currentAnimation->state()==QAbstractAnimation::Running){
        _currentAnimation->pause();
        _graphicsView->setRemoteMode(DeclarativeGraphicsView::Paused);
    }
}

void PorgySmallMultiplesTraceView::stop(){
    _goNextStep = false;
    if(_currentAnimation){
        _currentAnimation->stop();
    }

    //Animation is stopped allow user to edit configuration widget.
    _configurationWidget->setEnabled(true);
    _graphicsView->setRemoteMode(DeclarativeGraphicsView::Paused);//Set to the play button
}

void PorgySmallMultiplesTraceView::seekBackward(){    
    if(_currentState!=0){
        animate(_currentState-1);
    }
}

void PorgySmallMultiplesTraceView::seekForward(){    
    if(_currentState!= _states.stateNumber()-1){
        animate(_currentState+1);
    }else{
        stop();//Stop playing
    }
}

void PorgySmallMultiplesTraceView::seekFirst(){    
    if(_currentState!= 0){
        animate(0);
    }
}

void PorgySmallMultiplesTraceView::seekLast(){    
    if(_currentState!= _states.stateNumber() -1 ){
        animate(_states.stateNumber() -1 );
    }
}

void PorgySmallMultiplesTraceView::currentAnimationStateChanged(QAbstractAnimation::State newState, QAbstractAnimation::State ){
    //Animation will be deleted
    if(newState == QAbstractAnimation::Stopped){
        _currentAnimation->setCurrentTime(_currentAnimation->duration()-1);//Force to display the last step
        _currentAnimation->deleteLater();//Delete ended animation
        _currentAnimation=NULL;
        if(_goNextStep){//If we are playing all the video go next.
            seekForward();
        }else{
            //Stop the animation.
            stop();
        }
    }
}



//void PorgySmallMultiplesTraceView::itemSelected(int element){
//    animate(element);
//    zoomOnItem(element);
//}


void PorgySmallMultiplesTraceView::updateSmallMultipleLayout(){
    updateSmallMultiples(Update_Layout);
}

void PorgySmallMultiplesTraceView::displayAnimationWidget(bool displayWidget){    
    _graphicsView->setAnimationVisible(displayWidget);
}

void PorgySmallMultiplesTraceView::displaySmallMultipleWidget(bool displayWidget){    
    _graphicsView->setSmallMultiplesVisible(displayWidget);
}

void PorgySmallMultiplesTraceView::updateIntermediateStates(){
    //Regenerate state list
    _states.updateStateList(_configurationWidget->intermediateStates());
    updateSmallMultiples(Reset);
    //Force small multiples to redraw.
    //    draw();
    //Go to the first step.
    animate(0);
}


void PorgySmallMultiplesTraceView::draw(){
    _graphicsView->smallMultipleItem()->draw();//Force to draw the small multiples
    _graphicsView->animationItem()->draw();//Force to draw the graph
    _graphicsView->rulePreviewItem()->draw();//Force to update the rule preview
    _graphicsView->scene()->update();//Force to redraw the scene
}

void PorgySmallMultiplesTraceView::rulePreviewGeometryChanged(){
    RulePreviewGeometry geometry = _animationConfigurationWidget->rulePreviewGeometry();
    bool isVisible = geometry != No_Rule_Preview && _graphicsView->rulePreviewItem()->scene().getGlGraphComposite()!=NULL &&_graphicsView->rulePreviewItem()->scene().getGlGraphComposite()->getGraph() != NULL;
    _graphicsView->setRulePreviewVisible(isVisible);
    _graphicsView->setRulePreviewGeometry(geometry);
}

QPixmap PorgySmallMultiplesTraceView::snapshot(const QSize& s) const{
    if(_graphicsView->animationItem()->isVisible() && _graphicsView->smallMultipleItem()->isVisible()){

        //Compute the percent of each widget.
        float overviewPercent = 0.5f;
        //Divide the image in two parts.
        int overviewHeight = s.height()*overviewPercent;
        int videoHeight = s.height() - overviewHeight;

        //If we try to render the two elements at a same time don't take care of zoom and offsets.
        QImage smallMultipleImage = _graphicsView->smallMultipleItem()->createPixmap(QSize(s.width(),overviewHeight)).toImage();
        QImage videoImage = _graphicsView->animationItem()->createPixmap(QSize(s.width(),videoHeight)).toImage();
        //Generate the output image
        QImage ouptut(s,smallMultipleImage.format());
        //Copy small multiples pixels
        for(int x = 0 ; x < smallMultipleImage.width(); ++x){
            for(int y = 0 ; y < smallMultipleImage.height(); ++y){
                ouptut.setPixel(x,y,smallMultipleImage.pixel(x,y));
            }
        }
        //Copy video pixels
        for(int x = 0 ; x < videoImage.width(); ++x){
            for(int y = 0 ; y < videoImage.height(); ++y){
                //Shift y.
                ouptut.setPixel(x,overviewHeight + y,videoImage.pixel(x,y));
            }
        }
        return QPixmap::fromImage(ouptut);
    }else{
        if(_graphicsView->smallMultipleItem()->isVisible()){
            return _graphicsView->smallMultipleItem()->createPixmap(s);
        }else{
            return _graphicsView->animationItem()->createPixmap(s);
        }
    }
}

PLUGIN(PorgySmallMultiplesTraceView)
