#include <porgy/GraphElementValueModel.h>
#include <porgy/PropertyValueToQVariantManager.h>

#include <tulip/ForEach.h>
#include <tulip/PropertyInterface.h>

#include <vector>

using namespace std;
using namespace tlp;

std::string toLower(const std::string& data){
    std::string result=data;
    std::transform(data.begin(), data.end(),result.begin(),(int (*)(int))tolower);
    return result;
}

bool lessThan(tlp::PropertyInterface* p1, tlp::PropertyInterface* p2)
{
    return  toLower(p1->getName()) < toLower(p2->getName());
}

bool greaterThan(tlp::PropertyInterface* p1, tlp::PropertyInterface* p2){
    return toLower(p1->getName()) > toLower(p2->getName());
}

GraphElementValueModel::GraphElementValueModel(tlp::Graph* graph, tlp::ElementType eltType, unsigned int eltId,const std::vector<std::string>& propertiesToDisplay,QObject *parent):QAbstractTableModel(parent),_propertyDataConverter(new PropertyValueToQVariantManager()),_graph(graph),_displayType(eltType),_eltId(eltId)
{
    initPropertiesList(propertiesToDisplay);
}

GraphElementValueModel::GraphElementValueModel(tlp::Graph* graph, tlp::ElementType eltType, unsigned int eltId,QObject* parent ):QAbstractTableModel(parent),_propertyDataConverter(new PropertyValueToQVariantManager()),_graph(graph),_displayType(eltType),_eltId(eltId){
    vector<string> propertiesToDisplay;
    string propertyName;
    forEach(propertyName,graph->getProperties()){
        propertiesToDisplay.push_back(propertyName);
    }
    initPropertiesList(propertiesToDisplay);
}

void GraphElementValueModel::initPropertiesList(const vector<string>& propertiesToDisplay){
    beginResetModel();
    _propertyTable.clear();
    _propertyTable.reserve(propertiesToDisplay.size());
    if(_graph != NULL){
        for(vector<string>::const_iterator it = propertiesToDisplay.begin() ; it != propertiesToDisplay.end(); ++it){
            if(_graph->existProperty(*it)){
                PropertyInterface *property = _graph->getProperty(*it);
                _propertyTable.push_back(property);
            }
        }
    }
    endResetModel();
}

QVariant GraphElementValueModel::data(const QModelIndex& index, int role) const
{
    if(hasIndex(index.row(),index.column())){
        PropertyInterface* p = _propertyTable[index.row()];
        return _displayType == NODE?_propertyDataConverter->nodeData(node(_eltId),p,role):_propertyDataConverter->edgeData(edge(_eltId),p,role);
    }
    return QVariant();
}

bool GraphElementValueModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
    if(role != Qt::EditRole){
        return QAbstractTableModel::setData(index,value,role);
    }
    if(hasIndex(index.row(),index.column())){
        PropertyInterface* p = _propertyTable[index.row()];
        bool result = _displayType==NODE?_propertyDataConverter->setNodeData(value,node(_eltId),p,role):_propertyDataConverter->setEdgeData(value,edge(_eltId),p,role);
        if(result){
            emit dataChanged(index,index);
            return true;
        }
    }
    return false;
}


Qt::ItemFlags GraphElementValueModel::flags(const QModelIndex& index) const
{
    if(hasIndex(index.row(),index.column())){
        PropertyInterface* p = _propertyTable[index.row()];
        return _displayType==NODE?_propertyDataConverter->nodeFlags(node(_eltId),p):_propertyDataConverter->edgeFlags(edge(_eltId),p);
    }
    return Qt::ItemFlags();
}

int GraphElementValueModel::columnCount(const QModelIndex&) const
{
    return 1;
}

int GraphElementValueModel::rowCount(const QModelIndex&) const
{
    return _propertyTable.size();
}

QVariant GraphElementValueModel::headerData(int section, Qt::Orientation orientation, int role) const{
    if(role == Qt::DisplayRole){
        if(orientation == Qt::Vertical){
            return QVariant(QString::fromUtf8(_propertyTable[section]->getName().c_str()));
        }else{
            return QVariant((elementType()==NODE?tr("Node "):tr("Edge "))+QString::number(elementId()));
        }
    }
    return QAbstractTableModel::headerData(section,orientation,role);
}

void GraphElementValueModel::sort ( int , Qt::SortOrder order ){
    emit layoutAboutToBeChanged();
    qStableSort(_propertyTable.begin(),_propertyTable.end(),order==Qt::AscendingOrder?lessThan:greaterThan);
    emit layoutChanged();

}
