#include <porgy/ModelElementValueModel.h>
#include <porgy/PropertyValueToQVariantManager.h>
#include <porgy/PropertyValueToQVariantConverter.h>
#include <porgy/PortNodeQt.h>

#include <portgraph/PorgyConstants.h>

#include <tulip/StringProperty.h>
#include <tulip/BooleanProperty.h>
#include <tulip/Graph.h>

using namespace tlp;
using namespace std;

ModelElementValueModel::ModelElementValueModel(Graph* graph, ElementType elementType, unsigned int elementId, QObject* parent ):GraphElementValueModel(graph,elementType,elementId,vector<string>(),parent)
{
    //Init specific properties converter
    propertyDataConverterManager()->setDefaultNodePropertyConverter(BooleanProperty::propertyTypename,new PortNodeBooleanPropertyConverter(graph));

    //Node type
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::PORTNODE_TYPE,new PortNodeTypeNodePropertyConverter());
    initPropertiesList(buildPropertiesList());
}

vector<string> ModelElementValueModel::buildPropertiesList(){
    vector<string> propertiesNames;
    propertiesNames.push_back(PorgyConstants::STATE);
    propertiesNames.push_back(PorgyConstants::TAG);
    propertiesNames.push_back("viewBorderWidth");
    propertiesNames.push_back("viewColor");
    propertiesNames.push_back("viewBorderColor");
    //Search for boolean properties : viewSelection and P properties.
    PropertyInterface* property;
    forEach(property,graph()->getObjectProperties())
    {
        if(property->getTypename() == BooleanProperty::propertyTypename){
            propertiesNames.push_back(property->getName());
        }
    }
    switch(elementType()) {
    case NODE:
    {
        propertiesNames.push_back("viewLabel");
        propertiesNames.push_back("viewShape");
        propertiesNames.push_back(PorgyConstants::PORTNODE_TYPE);
    break;
    }
    case EDGE:
    {
        propertiesNames.push_back(PorgyConstants::NEW);
        propertiesNames.push_back("viewSrcAnchorSize");
        propertiesNames.push_back("viewSrcAnchorShape");
        propertiesNames.push_back("viewTgtAnchorSize");
        propertiesNames.push_back("viewTgtAnchorShape");
        break;
    }
    }
    return propertiesNames;
}
