#include <tulip/IntegerProperty.h>
#include <tulip/TlpQtTools.h>
#include <tulip/TulipMetaTypes.h>
#include <tulip/TulipViewSettings.h>

#include <portgraph/PortGraphModel.h>

#include <porgy/PropertyValueToQVariantConverter.h>

using namespace tlp;

PropertyValueToQVariantConverter::~PropertyValueToQVariantConverter(){

}

QVariant AbstractEnumNodePropertyEditor::value(unsigned int id,tlp::PropertyInterface* property,int role) const{
    if(role == Qt::DisplayRole){
        tlp::IntegerProperty* intProp = dynamic_cast<tlp::IntegerProperty*>(property);
        return label(intProp->getNodeValue(tlp::node(id)));
    }else{
        return QVariant();
    }
}
QVariant AbstractEnumNodePropertyEditor::defaultValue(tlp::PropertyInterface* property,int role) const{
    if(role == Qt::DisplayRole){
        tlp::IntegerProperty* intProp = dynamic_cast<tlp::IntegerProperty*>(property);
        return label(intProp->getNodeDefaultValue());
    }else{
        return QVariant();
    }
}

QVariant AbstractEnumEdgePropertyEditor::value(unsigned int id,tlp::PropertyInterface* property,int role) const{
    if(role == Qt::DisplayRole){
        tlp::IntegerProperty* intProp = dynamic_cast<tlp::IntegerProperty*>(property);
        return label(intProp->getEdgeValue(tlp::edge(id)));
    }else{
        return QVariant();
    }
}
QVariant AbstractEnumEdgePropertyEditor::defaultValue(tlp::PropertyInterface* property,int role) const{
    if(role == Qt::DisplayRole){
        tlp::IntegerProperty* intProp = dynamic_cast<tlp::IntegerProperty*>(property);
        return label(intProp->getEdgeDefaultValue());
    }else{
        return QVariant();
    }
}

QString AbstractGraphIdPropertyConverter::idToGraph(tlp::Graph* graph,const int& value)const{
    if(graph != NULL){
        tlp::Graph* child = (int)graph->getId() == value? graph : graph->getRoot()->getDescendantGraph(value);
        if(child != NULL){
            return tlp::tlpStringToQString(child->getName());
        }
    }
    return QString();
}

    QVariant NodeGraphIdPropertyConverter::value(unsigned int id,tlp::PropertyInterface* property,int role) const{
        if(role == Qt::DisplayRole){
            tlp::IntegerProperty* intProp = dynamic_cast<tlp::IntegerProperty*>(property);
            return idToGraph(intProp->getGraph(),intProp->getNodeValue(tlp::node(id)));
        }else{
            return QVariant();
        }
    }

    QVariant NodeGraphIdPropertyConverter::defaultValue(tlp::PropertyInterface* property,int role) const{
        if(role == Qt::DisplayRole){
            tlp::IntegerProperty* intProp = dynamic_cast<tlp::IntegerProperty*>(property);
            return idToGraph(intProp->getGraph(),intProp->getNodeDefaultValue());
        }else{
            return QVariant();
        }
    }

    QVariant EdgeGraphIdPropertyConverter::value(unsigned int id,tlp::PropertyInterface* property,int role) const{
        if(role == Qt::DisplayRole){
            tlp::IntegerProperty* intProp = dynamic_cast<tlp::IntegerProperty*>(property);
            return idToGraph(intProp->getGraph(),intProp->getEdgeValue(tlp::edge(id)));
        }else{
            return QVariant();
        }
    }
    QVariant EdgeGraphIdPropertyConverter::defaultValue(tlp::PropertyInterface* property,int role) const{
        if(role == Qt::DisplayRole){
            tlp::IntegerProperty* intProp = dynamic_cast<tlp::IntegerProperty*>(property);
            return idToGraph(intProp->getGraph(),intProp->getEdgeDefaultValue());
        }else{
            return QVariant();
        }
    }

    QVariant NodeShapePropertyEditor::value(unsigned int id,tlp::PropertyInterface* property,int role) const{
        if(role == Qt::DisplayRole || role == Qt::EditRole  ){
            tlp::IntegerProperty* p = dynamic_cast<tlp::IntegerProperty*>(property);
            return p == NULL? QVariant():QVariant::fromValue<tlp::NodeShape::NodeShapes>(static_cast<tlp::NodeShape::NodeShapes>(p->getNodeValue(tlp::node(id))));
        }else{
            return QVariant();
        }

    }
    QVariant NodeShapePropertyEditor::defaultValue(tlp::PropertyInterface* property,int role) const{
        if(role == Qt::DisplayRole || role == Qt::EditRole  ){
            tlp::IntegerProperty* p = dynamic_cast<tlp::IntegerProperty*>(property);
            return p == NULL? QVariant():QVariant::fromValue<tlp::NodeShape::NodeShapes>(static_cast<tlp::NodeShape::NodeShapes>(p->getNodeDefaultValue()));
        }else{
            return QVariant();
        }
    }
    bool NodeShapePropertyEditor::setValue(unsigned int id,tlp::PropertyInterface* property,const QVariant& data,int role) const{
        if(role == Qt::EditRole  ){
            tlp::IntegerProperty* p = dynamic_cast<tlp::IntegerProperty*>(property);
            p->setNodeValue(tlp::node(id),data.value<tlp::NodeShape::NodeShapes>());
            return true;
        }
        return false;
    }
    bool NodeShapePropertyEditor::setAllValue(tlp::PropertyInterface* property,const QVariant& data,int role) const{
        if(role == Qt::EditRole  ){
            tlp::IntegerProperty* p = dynamic_cast<tlp::IntegerProperty*>(property);
            p->setAllNodeValue(data.value<tlp::NodeShape::NodeShapes>());
            return true;
        }
        return false;
    }

    QVariant EdgeShapePropertyEditor::value(unsigned int id,tlp::PropertyInterface* property,int role) const{
        if(role == Qt::DisplayRole || role == Qt::EditRole  ){
            tlp::IntegerProperty* p = dynamic_cast<tlp::IntegerProperty*>(property);
            return p == NULL? QVariant():QVariant::fromValue<tlp::EdgeShape::EdgeShapes>(static_cast<tlp::EdgeShape::EdgeShapes>(p->getEdgeValue(tlp::edge(id))));
        }else{
            return QVariant();
        }

    }
    QVariant EdgeShapePropertyEditor::defaultValue(tlp::PropertyInterface* property,int role) const{
        if(role == Qt::DisplayRole || role == Qt::EditRole  ){
            tlp::IntegerProperty* p = dynamic_cast<tlp::IntegerProperty*>(property);
            return p == NULL? QVariant():QVariant::fromValue<tlp::EdgeShape::EdgeShapes>(static_cast<tlp::EdgeShape::EdgeShapes>(p->getEdgeDefaultValue()));
        }else{
            return QVariant();
        }
    }
    bool EdgeShapePropertyEditor::setValue(unsigned int id,tlp::PropertyInterface* property,const QVariant& data,int role) const{
        if(role == Qt::EditRole  ){
            tlp::IntegerProperty* p = dynamic_cast<tlp::IntegerProperty*>(property);
            p->setEdgeValue(tlp::edge(id),static_cast<tlp::EdgeShape::EdgeShapes>(data.value<tlp::EdgeShape::EdgeShapes>()));
            return true;
        }
        return false;
    }
    bool EdgeShapePropertyEditor::setAllValue(tlp::PropertyInterface* property,const QVariant& data,int role) const{
        if(role == Qt::EditRole  ){
            tlp::IntegerProperty* p = dynamic_cast<tlp::IntegerProperty*>(property);
            p->setAllEdgeValue(static_cast<tlp::EdgeShape::EdgeShapes>(data.value<tlp::EdgeShape::EdgeShapes>()));
            return true;
        }
        return false;
    }


        PortNodeBooleanPropertyConverter::PortNodeBooleanPropertyConverter(tlp::Graph* graph):NodeValueToQVariantConverter<tlp::BooleanProperty,bool>(),_graph(graph){
        }

        bool PortNodeBooleanPropertyConverter::setValue(unsigned int id,tlp::PropertyInterface* property,const QVariant& data,int role) const{
            if(role == Qt::EditRole){
                PortGraphModel model(_graph);
                PortNode *p = model.getPortNode(node(id));
                p->select(data.value<bool>(),static_cast<tlp::BooleanProperty*>(property));
                return true;
            }
            return false;
        }
