#include <porgy/PropertyValueToQVariantManager.h>
#include <porgy/PropertyValueToQVariantConverter.h>

#include <tulip/BooleanProperty.h>
#include <tulip/ColorProperty.h>
#include <tulip/DoubleProperty.h>
#include <tulip/GraphProperty.h>
#include <tulip/IntegerProperty.h>
#include <tulip/LayoutProperty.h>
#include <tulip/SizeProperty.h>
#include <tulip/StringProperty.h>
#include <tulip/TulipMetaTypes.h>

using namespace tlp;
using namespace std;

template<>
unsigned int qHash(const string& s){
    return qHash(QString(s.c_str()));
}

#define REGISTER_CONVERTER(map,KEY,converter) map[KEY]=converter;
#define REGISTER_NODE_DEFAULT_CONVERTER(PROPERTYTYPE,DATATYPE) REGISTER_CONVERTER(_defaultNodeValueConverters, PROPERTYTYPE::propertyTypename, (new NodeValueToQVariantConverter<PROPERTYTYPE,DATATYPE>()) )
#define REGISTER_EDGE_DEFAULT_CONVERTER(PROPERTYTYPE,DATATYPE) REGISTER_CONVERTER(_defaultEdgeValueConverters, PROPERTYTYPE::propertyTypename, (new EdgeValueToQVariantConverter<PROPERTYTYPE,DATATYPE>()) )
#define REGISTER_NODE_OVERLOAD_CONVERTER(PROPERTYNAME,CONVERTER) REGISTER_CONVERTER(_overloadedPropertyNodeValueConverters, PROPERTYNAME, CONVERTER )
#define REGISTER_EDGE_OVERLOAD_CONVERTER(PROPERTYNAME,CONVERTER) REGISTER_CONVERTER(_overloadedPropertyEdgeValueConverters, PROPERTYNAME, CONVERTER )

PropertyValueToQVariantManager::PropertyValueToQVariantManager()
{

    //Register default properties converter
    REGISTER_NODE_DEFAULT_CONVERTER(BooleanProperty,bool)
            REGISTER_NODE_DEFAULT_CONVERTER(ColorProperty,Color)
            REGISTER_NODE_DEFAULT_CONVERTER(DoubleProperty,double)
            REGISTER_NODE_DEFAULT_CONVERTER(GraphProperty,Graph*)
            REGISTER_NODE_DEFAULT_CONVERTER(IntegerProperty,int)
            REGISTER_NODE_DEFAULT_CONVERTER(LayoutProperty,Coord)
            REGISTER_NODE_DEFAULT_CONVERTER(SizeProperty,Size)
            REGISTER_NODE_DEFAULT_CONVERTER(StringProperty,string)
            REGISTER_NODE_DEFAULT_CONVERTER(BooleanVectorProperty,vector<bool>)
            REGISTER_NODE_DEFAULT_CONVERTER(ColorVectorProperty,vector<Color>)
            REGISTER_NODE_DEFAULT_CONVERTER(CoordVectorProperty,vector<Coord>)
            REGISTER_NODE_DEFAULT_CONVERTER(DoubleVectorProperty,vector<double>)
            REGISTER_NODE_DEFAULT_CONVERTER(IntegerVectorProperty,vector<int>)
            REGISTER_NODE_DEFAULT_CONVERTER(SizeVectorProperty,vector<Size>)
            REGISTER_NODE_DEFAULT_CONVERTER(StringVectorProperty,vector<string>)

            REGISTER_EDGE_DEFAULT_CONVERTER(BooleanProperty,bool)
            REGISTER_EDGE_DEFAULT_CONVERTER(ColorProperty,Color)
            REGISTER_EDGE_DEFAULT_CONVERTER(DoubleProperty,double)
            REGISTER_EDGE_DEFAULT_CONVERTER(GraphProperty,set<edge>)
            REGISTER_EDGE_DEFAULT_CONVERTER(IntegerProperty,int)
            REGISTER_EDGE_DEFAULT_CONVERTER(LayoutProperty,std::vector<Coord>)
            REGISTER_EDGE_DEFAULT_CONVERTER(SizeProperty,Size)
            REGISTER_EDGE_DEFAULT_CONVERTER(StringProperty,string)
            REGISTER_EDGE_DEFAULT_CONVERTER(BooleanVectorProperty,vector<bool>)
            REGISTER_EDGE_DEFAULT_CONVERTER(ColorVectorProperty,vector<Color>)
            REGISTER_EDGE_DEFAULT_CONVERTER(CoordVectorProperty,vector<Coord>)
            REGISTER_EDGE_DEFAULT_CONVERTER(DoubleVectorProperty,vector<double>)
            REGISTER_EDGE_DEFAULT_CONVERTER(IntegerVectorProperty,vector<int>)
            REGISTER_EDGE_DEFAULT_CONVERTER(SizeVectorProperty,vector<Size>)
            REGISTER_EDGE_DEFAULT_CONVERTER(StringVectorProperty,vector<string>)

            //Tulip special properties
            REGISTER_NODE_OVERLOAD_CONVERTER("viewShape",new NodeShapePropertyEditor);

            REGISTER_EDGE_OVERLOAD_CONVERTER("viewShape",new EdgeShapePropertyEditor);

}

void PropertyValueToQVariantManager::setDefaultNodePropertyConverter(const string& propertyTypeName,PropertyValueToQVariantConverter* converter){
    replaceConverter(_defaultNodeValueConverters,propertyTypeName,converter);
}

void PropertyValueToQVariantManager::setDefaultEdgePropertyConverter(const string& propertyTypeName,PropertyValueToQVariantConverter* converter){
    replaceConverter(_defaultEdgeValueConverters,propertyTypeName,converter);
}

void PropertyValueToQVariantManager::overloadNodePropertyConverter(const string& propertyName,PropertyValueToQVariantConverter* converter){
    replaceConverter(_overloadedPropertyNodeValueConverters,propertyName,converter);
}

void PropertyValueToQVariantManager::overloadEdgePropertyConverter(const string& propertyName,PropertyValueToQVariantConverter* converter){
    replaceConverter(_overloadedPropertyEdgeValueConverters,propertyName,converter);
}

void PropertyValueToQVariantManager::replaceConverter(QHash<string,PropertyValueToQVariantConverter*>& map,const string& key,PropertyValueToQVariantConverter* converter){
    QHash<string,PropertyValueToQVariantConverter*>::iterator it = map.find(key);
    if(it != map.end()){
        delete it.value();
        map.erase(it);
    }
    if(converter != NULL){
        map[key]=converter;
    }
}

PropertyValueToQVariantConverter* PropertyValueToQVariantManager::findConverter(QHash<string,PropertyValueToQVariantConverter*>& overloadedProperties,QHash<string,PropertyValueToQVariantConverter*>& defaultProperties,tlp::PropertyInterface* p){
    assert(p != NULL);
    //Search for overloaded properties
    PropertyValueToQVariantConverter* converter=NULL;
    if(!p->getName().empty()){
        converter = findConverter(overloadedProperties,string(p->getName()));
    }
    if(converter == NULL){
        //Search for default converter
        converter = findConverter(defaultProperties,p->getTypename());
    }
    return converter;
}



QVariant PropertyValueToQVariantManager::nodeData(tlp::node n,tlp::PropertyInterface* p,int role){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyNodeValueConverters,_defaultNodeValueConverters,p);
    return converter != NULL ? converter->value(n.id,p,role):QVariant();
}

QVariant PropertyValueToQVariantManager::defaultNodeData(tlp::PropertyInterface* p,int role){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyNodeValueConverters,_defaultNodeValueConverters,p);
    return converter != NULL ? converter->defaultValue(p,role):QVariant();
}

bool PropertyValueToQVariantManager::setNodeData(const QVariant& data,tlp::node n,tlp::PropertyInterface* p,int role){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyNodeValueConverters,_defaultNodeValueConverters,p);
    return converter != NULL ? converter->setValue(n.id,p,data,role):false;
}

bool PropertyValueToQVariantManager::setAllNodeData(const QVariant& data,tlp::PropertyInterface* p,int role){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyNodeValueConverters,_defaultNodeValueConverters,p);
    return converter != NULL ? converter->setAllValue(p,data,role):false;
}

QVariant PropertyValueToQVariantManager::edgeData(tlp::edge e,tlp::PropertyInterface* p,int role){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyEdgeValueConverters,_defaultEdgeValueConverters,p);
    return converter != NULL ? converter->value(e.id,p,role):QVariant();
}

QVariant PropertyValueToQVariantManager::defaultEdgeData(PropertyInterface* p,int role){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyEdgeValueConverters,_defaultEdgeValueConverters,p);
    return converter != NULL ? converter->defaultValue(p,role):QVariant();
}

bool PropertyValueToQVariantManager::setEdgeData(const QVariant& data, tlp::edge e, tlp::PropertyInterface* p, int role){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyEdgeValueConverters,_defaultEdgeValueConverters,p);
    return converter != NULL ? converter->setValue(e.id,p,data,role):false;
}

bool PropertyValueToQVariantManager::setDefaultEdgeData(const QVariant& data,tlp::PropertyInterface* p,int role){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyEdgeValueConverters,_defaultEdgeValueConverters,p);
    return converter != NULL ? converter->setAllValue(p,data,role):false;
}

PropertyValueToQVariantConverter* PropertyValueToQVariantManager::findConverter(const QHash<string,PropertyValueToQVariantConverter*>& map,const std::string& key)const {
    QHash<string,PropertyValueToQVariantConverter*>::const_iterator it = map.find(key);
    return it == map.end() ? NULL:it.value();
}

Qt::ItemFlags PropertyValueToQVariantManager::nodeFlags(tlp::node n,tlp::PropertyInterface* p){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyNodeValueConverters,_defaultNodeValueConverters,p);
    return converter != NULL ? converter->flags(n.id,p):Qt::ItemFlags();
}

Qt::ItemFlags PropertyValueToQVariantManager::nodeDefaultFlags(tlp::PropertyInterface* p){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyNodeValueConverters,_defaultNodeValueConverters,p);
    return converter != NULL ? converter->defaultFlags(p):Qt::ItemFlags();
}

Qt::ItemFlags PropertyValueToQVariantManager::edgeFlags(edge e,tlp::PropertyInterface* p){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyEdgeValueConverters,_defaultEdgeValueConverters,p);
    return converter != NULL ? converter->flags(e.id,p):Qt::ItemFlags();
}

Qt::ItemFlags PropertyValueToQVariantManager::edgeDefaultFlags(tlp::PropertyInterface* p){
    PropertyValueToQVariantConverter* converter=findConverter(_overloadedPropertyEdgeValueConverters,_defaultEdgeValueConverters,p);
    return converter != NULL ? converter->defaultFlags(p):Qt::ItemFlags();
}
