#include <porgy/RuleElementValueModel.h>
#include <porgy/PropertyValueToQVariantManager.h>
#include <porgy/PropertyValueToQVariantConverter.h>
#include <porgy/PortNodeQt.h>

#include <portgraph/PorgyConstants.h>

#include <tulip/Graph.h>
#include <tulip/StringProperty.h>
#include <tulip/BooleanProperty.h>
#include <tulip/IntegerProperty.h>

using namespace tlp;
using namespace std;

/**
 * @brief The TransformationTypePropertyConverter class converter for PorgyConstants::RuleSide enum type.
 */
class RuleSideNodePropertyConverter : public AbstractEnumNodePropertyEditor{
public:

    QString label(const int& value)const{

        PorgyConstants::RuleSide tType = static_cast<PorgyConstants::RuleSide>(value);
        switch(tType){
        case PorgyConstants::SIDE_BRIDGE:
            return QString("Bridge");
            break;
        case PorgyConstants::SIDE_LEFT:
            return QString("Left");
            break;
        case PorgyConstants::SIDE_RIGHT:
            return QString("Right");
            break;
        case PorgyConstants::SIDE_BRIDGE_OPP:
            return QString("Bridge opp");
            break;
        case PorgyConstants::SIDE_BRIDGE_PORT:
            return QString("Bridge port");
            break;
        case PorgyConstants::NO_SIDE:
            return QString("None");
            break;
    }

        return QString();
    }

};

class RuleSideEdgePropertyConverter : public AbstractEnumEdgePropertyEditor{
public:

    QString label(const int& value)const{

        PorgyConstants::RuleSide tType = static_cast<PorgyConstants::RuleSide>(value);
        switch(tType){
        case PorgyConstants::SIDE_BRIDGE:
            return QString("Bridge");
            break;
        case PorgyConstants::SIDE_LEFT:
            return QString("Left");
            break;
        case PorgyConstants::SIDE_RIGHT:
            return QString("Right");
            break;
        case PorgyConstants::SIDE_BRIDGE_OPP:
            return QString("Bridge opp");
            break;
        case PorgyConstants::SIDE_BRIDGE_PORT:
            return QString("Bridge port");
            break;
        case PorgyConstants::NO_SIDE:
            return QString("None");
            break;
    }

        return QString();
    }

};

RuleElementValueModel::RuleElementValueModel(Graph* graph, ElementType elementType, unsigned int elementId, QObject* parent ):GraphElementValueModel(graph,elementType,elementId,vector<string>(),parent)
{
    //Avoid modification on RULE_SIDE property
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::RULE_SIDE,new RuleSideNodePropertyConverter());
    propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::RULE_SIDE,new RuleSideEdgePropertyConverter());
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::PORTNODE_TYPE,new PortNodeTypeNodePropertyConverter());
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::M, new UneditableNodeValueToQVariantConverter<tlp::BooleanProperty,bool>());
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::NEW, new UneditableNodeValueToQVariantConverter<tlp::BooleanProperty,bool>());
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::ARITY, new NodeValueToQVariantConverter<tlp::IntegerProperty,unsigned int>());
    propertyDataConverterManager()->overloadNodePropertyConverter("viewSelection", new PortNodeBooleanPropertyConverter(graph));
    initPropertiesList(buildPropertiesList());

}

vector<string> RuleElementValueModel::buildPropertiesList(){
    vector<string> propertiesNames;
    propertiesNames.push_back(PorgyConstants::STATE);
    propertiesNames.push_back("viewColor");
    propertiesNames.push_back("viewBorderColor");
    propertiesNames.push_back("viewBorderWidth");
    switch(elementType()) {
    case NODE:
        propertiesNames.push_back("viewLabel");
        propertiesNames.push_back("viewSelection");
        propertiesNames.push_back(PorgyConstants::RULE_SIDE);
        propertiesNames.push_back(PorgyConstants::PORTNODE_TYPE);
        propertiesNames.push_back(PorgyConstants::M);
        propertiesNames.push_back(PorgyConstants::NEW);
        propertiesNames.push_back(PorgyConstants::ARITY);
        break;
    case EDGE:
        propertiesNames.push_back(PorgyConstants::RULE_SIDE);
        break;
    }

    return propertiesNames;
}
