#include <portgraph/Trace.h>
#include <portgraph/PorgyConstants.h>

#include <porgy/TraceElementValueModel.h>
#include <porgy/PropertyValueToQVariantManager.h>
#include <porgy/PropertyValueToQVariantConverter.h>

#include <tulip/IntegerProperty.h>
#include <tulip/StringProperty.h>
#include <tulip/TlpQtTools.h>

#include <tulip/Graph.h>

using namespace tlp;
using namespace std;

/**
 * @brief The TransformationTypePropertyConverter class converter for Trace::TransformationType enum type.
 */
class TransformationTypePropertyConverter : public AbstractEnumEdgePropertyEditor{
public:

    QString label(const int& value)const{
        Trace::TransformationType tType = static_cast<Trace::TransformationType>(value);
        switch(tType){
        case Trace::Simple_Transformation:
            return QString("Simple Transformation");
            break;
        case Trace::Probabilistic_Transformation:
            return QString("Probabilistic Transformation");
            break;
        case Trace::Strategy_Transformation:
            return QString("Strategy");
            break;
        case Trace::Focus_Transformation:
            return QString("Focus");
            break;
        case Trace::Ban_Transformation:
            return QString("Ban");
            break;
        }
        return QString();
    }

    Qt::ItemFlags defaultFlags(tlp::PropertyInterface*){
        return Qt::ItemFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable );
    }

};

/**
 * @brief The ParallelTypePropertyConverter class is and editor for the Trace::TransformationParallelInformation enum type.
 */
class ParallelTypePropertyConverter : public AbstractEnumEdgePropertyEditor{
public:
    QString label(const int& value)const{
        Trace::TransformationParallelInformation tType = static_cast<Trace::TransformationParallelInformation>(value);
        switch(tType){
        case Trace::No_parallel:
            return QString("No parallel");
            break;
        case Trace::Lazy_Parallel:
            return QString("Lazy parallel");
            break;
        case Trace::Real_Parallel:
            return QString("Real parallel");
            break;
        }
        return QString();
    }

    Qt::ItemFlags defaultFlags(tlp::PropertyInterface*){
        return Qt::ItemFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable );
    }
};

/**
 * @brief The TraceNodeTypePropertyConverter class is an editor for the values of the Trace::NodeType enum.
 */
class TraceNodeTypePropertyConverter : public AbstractEnumNodePropertyEditor{
public:
    QString label(const int& value)const{
        Trace::NodeType tType = static_cast<Trace::NodeType>(value);
        switch(tType){
        case Trace::Failure_Node:
            return QString("Failure");
            break;
        case Trace::Model_Node:
            return QString("Graph");
            break;
        }
        return QString();
    }

    Qt::ItemFlags defaultFlags(tlp::PropertyInterface*){
        return Qt::ItemFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable );
    }
};



TraceElementValueModel::TraceElementValueModel(Graph* graph, ElementType elementType, unsigned int elementId, QObject* parent ):GraphElementValueModel(graph,elementType,elementId,vector<string>(),parent)
{
    //By default all string properties will be unmodifiable
    propertyDataConverterManager()->setDefaultEdgePropertyConverter(StringProperty::propertyTypename,new UneditableEdgeValueToQVariantConverter<StringProperty,string>);
    propertyDataConverterManager()->setDefaultNodePropertyConverter(StringProperty::propertyTypename,new UneditableEdgeValueToQVariantConverter<StringProperty,string>);
    //Overload properties nodes converters
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::TYPE_PROPERTY,new TraceNodeTypePropertyConverter);
    propertyDataConverterManager()->overloadNodePropertyConverter(PorgyConstants::TRANSFORMATION_RULE_PROPERTY,new NodeGraphIdPropertyConverter());

    //Overload properties edges converters
    propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::TRANSFORMATION_PARALLEL_INFORMATION_PROPERTY,new ParallelTypePropertyConverter);
    propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::TYPE_PROPERTY,new TransformationTypePropertyConverter());
    propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::TRANSFORMATION_RULE_PROPERTY,new EdgeGraphIdPropertyConverter());
    propertyDataConverterManager()->overloadEdgePropertyConverter(PorgyConstants::TRANSFORMATION_INSTANCE_PROPERTY,new EdgeGraphIdPropertyConverter());

    //Set properties to dipslay
    initPropertiesList(buildPropertiesList());
}

vector<string> TraceElementValueModel::buildPropertiesList(){
    vector<string> propertiesNames;
    Trace traceGraph(graph());
    //The type property is used for both nodes and edges
    propertiesNames.push_back(PorgyConstants::TYPE_PROPERTY);
    if(elementType() == EDGE){
        propertiesNames.push_back(PorgyConstants::TRANSFORMATION_PPROPERTYNAME_PROPERTY);
        propertiesNames.push_back(PorgyConstants::TRANSFORMATION_BANPROPERTYNAME_PROPERTY);

        Trace::TransformationType transformation = traceGraph.getTransformationType(edge(elementId()));
        if((transformation == Trace::Simple_Transformation)||(transformation==Trace::Probabilistic_Transformation)){
            propertiesNames.push_back(PorgyConstants::TRANSFORMATION_RULE_PROPERTY);
            propertiesNames.push_back(PorgyConstants::TRANSFORMATION_INSTANCE_PROPERTY);
            propertiesNames.push_back(PorgyConstants::TRANSFORMATION_PARALLEL_INFORMATION_PROPERTY);
            if(transformation==Trace::Probabilistic_Transformation)
                propertiesNames.push_back(PorgyConstants::TRANSFORMATION_PROBA_PROPERTY);
        }else if(transformation == Trace::Strategy_Transformation){
            propertiesNames.push_back(PorgyConstants::TRANSFORMATION_STRATEGY_PROPERTY);
        }else if(transformation == Trace::Focus_Transformation){
            propertiesNames.push_back(PorgyConstants::TRANSFORMATION_RULE_PROPERTY);
            propertiesNames.push_back(PorgyConstants::TRANSFORMATION_INSTANCE_PROPERTY);
            propertiesNames.push_back(PorgyConstants::TRANSFORMATION_PARALLEL_INFORMATION_PROPERTY);
        }
    }else{
        propertiesNames.push_back("viewLabel");
        if(traceGraph.isFailureNode(node(elementId()))){
            propertiesNames.push_back(PorgyConstants::TRANSFORMATION_RULE_PROPERTY);
        }
    }
    return propertiesNames;
}
