#include <porgy/elementinformationinteractorcomponent.h>
#include <porgy/TraceElementValueModel.h>
#include <porgy/ModelElementValueModel.h>
#include <porgy/RuleElementValueModel.h>

#include <portgraph/PorgyTlpGraphStructure.h>
#include <portgraph/PortGraphRule.h>

#include <tulip/View.h>
#include <tulip/GlMainWidget.h>
#include <tulip/GlGraphComposite.h>

#include <QMouseEvent>
#include <QEvent>

using namespace tlp;

ElementInformationInteractorComponent::ElementInformationInteractorComponent()
{
}

QAbstractItemModel* ElementInformationInteractorComponent::buildModel(ElementType elementType, unsigned int elementId, QObject *parent)const{
    Graph* graph = view()->graph();
    GraphElementValueModel *model =  NULL;
    if(PorgyTlpGraphStructure::isTraceGraph(graph)){
        model = new TraceElementValueModel(graph,elementType,elementId,parent);
    }else if(PorgyTlpGraphStructure::isModelGraph(graph)){
        model = new ModelElementValueModel(graph,elementType,elementId,parent);
    }else if(PorgyTlpGraphStructure::isRuleGraph(graph)){
        model = new RuleElementValueModel(graph,elementType,elementId,parent);
    }
    if(model != NULL){
        model->sort(0);
    }
    return model;
}

bool ElementInformationInteractorComponent::eventFilter(QObject *widget, QEvent *e) {
    QMouseEvent *qMouseEv = static_cast<QMouseEvent *>(e);
    if (((e->type() == QEvent::MouseButtonPress)&&(qMouseEv->button() == Qt::LeftButton))||(e->type()==QEvent::MouseMove)) {
        GlMainWidget *glMainWidget = static_cast<GlMainWidget *>(widget);
        Graph *g = glMainWidget->getScene()->getGlGraphComposite()->getInputData()->getGraph();
        if(PorgyTlpGraphStructure::isRuleGraph(g)) {
            SelectedEntity pickedEntity;
            glMainWidget->pickNodesEdges(qMouseEv->x(), qMouseEv->y(), pickedEntity, NULL, true,true);
            PortGraphRule pg(g);
            IntegerProperty *rule_side = pg.getSideProperty();
            edge ed = pickedEntity.getEdge();
            if(ed.isValid()) {
                if(rule_side->getEdgeValue(ed)==PorgyConstants::SIDE_BRIDGE) {
                    e->ignore();
                    return true;
                }
            }
            node n = pickedEntity.getNode();
            if(n.isValid()) {
                if(rule_side->getNodeValue(n)==PorgyConstants::SIDE_BRIDGE) {
                    e->ignore();
                    return true;
                }
            }
        }
    }
    return MouseShowElementInfos::eventFilter(widget, e);
}
