#ifndef GRAPHELEMENTVALUEMODEL_H
#define GRAPHELEMENTVALUEMODEL_H
#include <QAbstractTableModel>
#include <QVector>

#include <tulip/Graph.h>

class PropertyValueToQVariantManager;
/**
 * @brief The GraphElementValueModel class Displays properties for given element according to it's type and id.
 * To filter the displayed properties give the name of the properties to visualize
 */
class GraphElementValueModel : public QAbstractTableModel
{

    PropertyValueToQVariantManager* _propertyDataConverter;
    tlp::Graph* _graph;
    tlp::ElementType _displayType;
    unsigned int _eltId;
    QVector<tlp::PropertyInterface*> _propertyTable;
public:
    /**
     * @brief GraphElementValueModel Display only properties with the name given in the propertiesToDisplay parameter.
     * @param graph
     * @param eltType
     * @param eltId
     * @param propertiesToDisplay
     * @param parent
     */
    GraphElementValueModel(tlp::Graph* graph, tlp::ElementType eltType, unsigned int eltId,const std::vector<std::string>& propertiesToDisplay,QObject* parent = NULL);
    /**
     * @brief GraphElementValueModel Displays all properties available in the graph.
     * @param graph
     * @param eltType
     * @param eltId
     * @param parent
     */
    GraphElementValueModel(tlp::Graph* graph, tlp::ElementType eltType, unsigned int eltId,QObject* parent = NULL);


    QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
    int columnCount(const QModelIndex& parent = QModelIndex()) const;
    int rowCount(const QModelIndex& parent = QModelIndex()) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
    bool setData( const QModelIndex & index, const QVariant & value, int role = Qt::EditRole );
    Qt::ItemFlags flags ( const QModelIndex & index ) const;
    void sort ( int column, Qt::SortOrder order = Qt::AscendingOrder );


    tlp::ElementType elementType()const{
        return _displayType;
    }

    unsigned int elementId()const{
        return _eltId;
    }

    tlp::Graph* graph()const{
        return _graph;
    }

    PropertyValueToQVariantManager* propertyDataConverterManager(){
        return _propertyDataConverter;
    }

protected:
    void initPropertiesList(const std::vector<std::string>& propertiesToDisplay);
};

#endif // GRAPHELEMENTVALUEMODEL_H
