#ifndef PORTNODEQTMODEL_H
#define PORTNODEQTMODEL_H

#include <QAbstractListModel>
#include <QVector>
#include <QSet>

#include <porgy/PortNodeQt.h>

#include <tulip/ForEach.h>
#include <tulip/TlpQtTools.h>
#include <tulip/Observable.h>

#include <portgraph/PortNode.h>
#include <portgraph/Bridge.h>

namespace tlp{
class Graph;
}

/**
  * @brief Simple list model for PortNodes to use with QListView.
  **/
class PortNodeQtModel : public QAbstractListModel, public tlp::Observable
{
    /**
     * @brief _portNodesList is the list of port nodes available
     */
    QVector<PortNodeQt *> _portNodesList;
    /**
     * @brief _rules_root The rule root graph. Listen this graph for node creation to automatically update the Portnode list.
     */
    tlp::Graph* _rules_root;
    /**
     * @brief _rules_root The models root graph. Listen this graph for node creation to automatically update the Portnode list.
     */
    tlp::Graph* _models_root;

    /**
     * @brief _needToReloadData marked true if data have changed and need to be updated.
     */
    bool _needToReloadData;
public:
    PortNodeQtModel(QObject* parent = NULL);
    ~PortNodeQtModel();

    void initPortNodesList(tlp::Graph* graph);

    void addPortNode(const PortNodeQt& prtNode);

    int rowCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;

    PortNodeQt *portNodeFromIndex(const QModelIndex &index)const;
    QList<PortNodeQt *> portNodesFromIndexes(const QModelIndexList& indexes)const;

    QStringList mimeTypes();
    QMimeData* mimeData(const QModelIndexList &indexes) const;

protected:

    void treatEvent(const tlp::Event &);
    void treatEvents(const std::vector<tlp::Event> &events);

private:

    void updateData();
    void clear();

    void addPortNode(PortNodeQt *portNode);

    template <typename TYPE>
    void loadOneGraph(const TYPE &pg){
        PortNode *pn;
        forEach(pn, pg.getPortNodes()) {
            if(!Bridge::isBridge(pn->getCenter(),pg.getGraph())){
                PortNodeQt * pnqt = new PortNodeQt(pn);
                //Check if the port node already exist in the list.
                PortNodeQt* port;
                bool add = true;
                foreach(port, _portNodesList){

                    //Check if the port node is not already in the list.
                    if(port->isSimilarTo(*pnqt)){
                        //The port was found no need to add it again
                        add = false;
                        //Treat the next port
                        break;
                    }
                }
                if(add){
                    //The port was not found add it in the list
                    addPortNode(pnqt);
                }else{
                    delete pnqt;
                }
            }
        }
    }
};

#endif // PORTNODEQTMODEL_H
