#ifndef GRAPHSNAPSHOTMANAGER_H
#define GRAPHSNAPSHOTMANAGER_H

#include <QObject>
#include <QPixmap>

#include <tulip/Observable.h>
#include <tulip/PropertyInterface.h>
#include <tulip/GlGraphRenderingParameters.h>
#include <tulip/Color.h>

#include <map>

namespace tlp{
    class Graph;
    class GlMetaNodeRenderer;
}

class GraphSnapshotManager : public QObject, tlp::Observable
{
    Q_OBJECT

    /** Graph observation variables */
    mutable std::map<tlp::Graph*,QPixmap> _previews;
    mutable std::map<tlp::PropertyInterface*,std::set<tlp::Graph*> > _oberverList;
    std::set<tlp::Graph*> _toUpdate;

    /** Snapshots visual parameters */
    QSize _snapshotSize;
    tlp::Color _backgroundColor;
    tlp::GlGraphRenderingParameters _renderingParameters;
    tlp::GlMetaNodeRenderer *_metanodeRenderer;

public:
    GraphSnapshotManager(QObject* parent=NULL);
    ~GraphSnapshotManager();

    /**
      * @brief Take a snapshot of a graph.
      **/
    static QPixmap takeSingleSnapshot(tlp::Graph* graph,const QSize& size ,const tlp::Color& backgroundColor = tlp::Color(255,255,255) , const tlp::GlGraphRenderingParameters& =tlp::GlGraphRenderingParameters(),tlp::GlMetaNodeRenderer* renderer = NULL);

    /**
      * @brief Return the snapshot of a graph. If the snapshot was already computed return the last snapshot taken. The snapshot is valid until a graph visual parameter will be updated or if the graph structure change.
      * To stop listening graph modification use the removeSnapshot function
      * If the graph snapshot changed the snapshotChanged signal is send.
      **/
    QPixmap takeSnapshot(tlp::Graph* graph);
    void removeSnapshot(tlp::Graph* graph);

    void clear();

    QSize previewSize()const{
        return _snapshotSize;
    }
    void setPreviewSize(const QSize& newSize);

    void setRenderingParameters(const tlp::GlGraphRenderingParameters& renderingParameters);
    tlp::GlGraphRenderingParameters renderingParameters()const{
        return _renderingParameters;
    }

    tlp::Color backgroundColor()const{
        return _backgroundColor;
    }
    void setBackgroundColor(const tlp::Color& color);

    tlp::GlMetaNodeRenderer* metanodeRenderer()const{
        return _metanodeRenderer;
    }
    void setMetanodeRenderer(tlp::GlMetaNodeRenderer* renderer);

    void treatEvents(const std::vector<tlp::Event> &events);
    void treatEvent(const tlp::Event &);

private:

    void observe(tlp::Graph*);
    void stopObserving(tlp::Graph*);

    std::set<std::string> generateVisualPropertiesNames(tlp::Graph* graph)const;

signals :
    void snapshotChanged(tlp::Graph*);
    void allSnapshotsChanged();
};

#endif // GRAPHSNAPSHOTMANAGER_H
