#include <porgy/pluginparametersconfigurationdialog.h>

#include "ui_pluginparametersconfigurationdialog.h"

#include <tulip/TulipItemDelegate.h>
#include <tulip/ParameterListModel.h>
#include <tulip/TulipMetaTypes.h>

using namespace tlp;

PluginParametersConfigurationDialog::PluginParametersConfigurationDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::PluginParametersConfigurationDialog),_defalutValues(NULL),_graph(NULL),_parametersModel(NULL)
{
    ui->setupUi(this);
    ui->tableView->setItemDelegate(new tlp::TulipItemDelegate(ui->tableView));
}

PluginParametersConfigurationDialog::~PluginParametersConfigurationDialog()
{
    delete ui;
    delete _defalutValues;
}

tlp::DataSet PluginParametersConfigurationDialog::values()const{
    if(_parametersModel != NULL){
        return _parametersModel->parametersValues();
    }else{
        return tlp::DataSet();
    }
}
void PluginParametersConfigurationDialog::setDefaultValues(const tlp::DataSet& values){
    tlp::DataSet* oldParam = _defalutValues;
    _defalutValues = new tlp::DataSet(values);
    if(_parametersModel != NULL){
        _parametersModel->setParametersValues(*_defalutValues);
    }

    delete oldParam;
}

void PluginParametersConfigurationDialog::setParametersList(const tlp::ParameterDescriptionList& parametersList){
    _parameters = parametersList;
    updateModel();
}

void PluginParametersConfigurationDialog::setGraph(tlp::Graph* graph){
    _graph = graph;
    updateModel();
}

void PluginParametersConfigurationDialog::updateModel(){
    tlp::ParameterListModel *oldModel = _parametersModel;

    _parametersModel = new tlp::ParameterListModel(_parameters,_graph,ui->tableView);
    if(_defalutValues != NULL){
        _parametersModel->setParametersValues(*_defalutValues);
    }
    ui->tableView->setModel(_parametersModel);
    ui->tableView->resizeColumnsToContents();

    if(oldModel != NULL){
        oldModel->deleteLater();
    }
}

DataSet PluginParametersConfigurationDialog::getParameters(QWidget* parent,const QString& title,const tlp::ParameterDescriptionList& parameters, bool &ok, tlp::DataSet* defaultValues,tlp::Graph* graph){
    PluginParametersConfigurationDialog dialog(parent);
    dialog.setWindowTitle(title);
    dialog.setParametersList(parameters);
    dialog.setGraph(graph);
    bool deleteDataSet = false;
    if(defaultValues == NULL){
        defaultValues = new DataSet();
        parameters.buildDefaultDataSet(*defaultValues,graph);
    }
    dialog.setDefaultValues(*defaultValues);
    if(dialog.exec() == QDialog::Accepted){
        ok = true;
        if(deleteDataSet){
            delete defaultValues;
        }
        return dialog.values();
    }else{
        ok = false;
        if(deleteDataSet){
            delete defaultValues;
        }
        return DataSet();
    }
}
