/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <tulip/IntegerProperty.h>

#include <portgraph/Port.h>
#include <portgraph/PorgyConstants.h>
#include <portgraph/PortNode.h>
#include <portgraph/PortGraph.h>
#include <portgraph/PorgyTlpGraphStructure.h>

using namespace std;

Port::Port(const tlp::node n, const PortNode *pp):PortBase(n,pp){

}

const PortNode* Port::getOneConnectedPortNode() const{
    return connectedPorts.empty()?NULL:connectedPorts[0]->getParentPortNode();
}

/****************
 * Test if two ports are equivalent
 *two ports are equivalent iff same name, same state and same degree
 */
bool Port::check(const PortPtr rule_port, const PortPtr modele_port, bool exact) {
#ifndef NDEBUG
    cerr << __PRETTY_FUNCTION__ << ": checking model port " << modele_port->getNode() << " and rule port " << rule_port->getNode() << endl;
#endif

        if(rule_port->getName() == modele_port->getName()) {
            if(exact) { //graph/graph matching
                if(rule_port->deg() == modele_port->deg()) {
                    if (rule_port->getState() == modele_port->getState()) {
                        return true;
                    }
#ifndef NDEBUG
                    else
                        cerr << __PRETTY_FUNCTION__ << ": State problem (model): " << modele_port->getState()  << " and (rule) " << rule_port->getState() << endl;
#endif
                }
#ifndef NDEBUG
                else
                    cerr << __PRETTY_FUNCTION__ << ": Problem with port degrees (model): " << modele_port->deg()  << " and (rule)" << rule_port->deg() << endl;
#endif
            }
            if(rule_port->getState() == PorgyConstants::STATE_NOTUSED) {
                if(rule_port->getArity() <= modele_port->deg()) {
                    return true;
                }
#ifndef NDEBUG
                else
                    cerr << __PRETTY_FUNCTION__ << ": Problem with port degrees (model with state not used): " << modele_port->deg()  << " and (rule)" << rule_port->getArity() << endl;
#endif
            }
            else {  //we have a state to check
                if(rule_port->getArity() == modele_port->deg()) {
                    if(rule_port->getState() == modele_port->getState()) {
                        return true;
                    }
#ifndef NDEBUG
                    else
                        cerr << __PRETTY_FUNCTION__ << ": State problem (model): " << modele_port->getState()  << " and (rule) " << rule_port->getState() << endl;
#endif
                }
#ifndef NDEBUG
                else
                    cerr << __PRETTY_FUNCTION__ << ": Problem with port degrees (model): " << modele_port->deg()  << " and (rule)" << rule_port->getArity() << endl;
#endif
            }
        }
#ifndef NDEBUG
        else
            cerr << __PRETTY_FUNCTION__ << ": Not the same name (model): " << modele_port->getName()  << " and (rule) " << rule_port->getName() << endl;
#endif

    return false;
}

unsigned int Port::getArity() const {
#ifndef NDEBUG
    if(!PorgyTlpGraphStructure::isRuleGraph(getParentPortNode()->getParentPortGraph()->getGraph()))
        std::cerr << __PRETTY_FUNCTION__ << " WARNING: graph is not a rule" << std::endl;
#endif
    tlp::IntegerProperty *arity = getParentPortNode()->getParentPortGraph()->getGraph()->getProperty<tlp::IntegerProperty>(PorgyConstants::ARITY);
    return static_cast<unsigned int>(arity->getNodeValue(getNode()));
}

unsigned int Port::deg() const {
    assert(getParentPortNode()->getParentPortGraph()->getGraph()->isElement(getNode()));
    return getParentPortNode()->getParentPortGraph()->getGraph()->deg(getNode())-1;
}
