/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * last modification : $Id:
 */
#include <tulip/ForEach.h>
#include <tulip/StringProperty.h>
#include <tulip/ColorProperty.h>

#include <portgraph/PortNodeBase.h>
#include <portgraph/PortGraph.h>
#include <portgraph/PorgyConstants.h>

using namespace std;
using namespace tlp;

//constants for drawing the PortNode
const int PortNodeBase::sizeRate = 1;

PortNodeBase::PortNodeBase(PortGraph *pg, const node &n):centerNode(n),ParentPortGraph(pg) {
   assert(PortNodeBase::isCenter(n, pg->getGraph()));
}

//renvoie le label du centre
string PortNodeBase::getName() const {
        return ParentPortGraph->getGraph()->getProperty<StringProperty>("viewLabel")->getNodeValue(centerNode);
}

Color PortNodeBase::getColor() const {
        return ParentPortGraph->getGraph()->getProperty<ColorProperty>("viewColor")->getNodeValue(centerNode);
}

tlp::IntegerProperty *PortNodeBase::getTypeProperty() const {
    return getTypeProperty(getParentPortGraph()->getGraph());
}

//static method
bool PortNodeBase::isCenter(const tlp::node n, tlp::Graph *g) {
	assert(g->isElement(n));
    return (PortNodeBase::getTypeProperty(g)->getNodeValue(n)==PorgyConstants::CENTER);

}

//change le alpha des sommets centraux
void PortNodeBase::setCenterAlphaValue(ColorProperty *colorProperty) const {
	Color col = colorProperty->getNodeValue(centerNode);
        col.setA(PorgyConstants::CENTER_ALPHA);
	colorProperty->setNodeValue(centerNode,col);
}

//rend invisible les arêtes
void PortNodeBase::hidePortNodeEdges(tlp::ColorProperty *colorProperty) const {
        ColorProperty *BorderColor = ParentPortGraph->getGraph()->getProperty<ColorProperty>("viewBorderColor");

	edge e;
	forEach(e, ParentPortGraph->getGraph()->getInOutEdges(centerNode)) {
		Color col = colorProperty->getEdgeValue(e);
		col.setA(PorgyConstants::PORTNODEEDGE_ALPHA);
		colorProperty->setEdgeValue(e,col);
		BorderColor->setEdgeValue(e,col);
	}
}

void PortNodeBase::setShape() const {
        ParentPortGraph->getGraph()->getProperty<IntegerProperty>("viewShape")->setNodeValue(centerNode, PorgyConstants::CENTER_SHAPE);
}
