/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _BRIDGE_H
#define _BRIDGE_H

#include <portgraph/PortNodeBase.h>
#include <portgraph/Port.h>

class BridgePort;
class PortNode;
class PortGraphRule;

class Bridge:public PortNodeBase {

    std::map<tlp::node, tlp::node> container_left_bridge;	//liaison entre node port du bridge et port de la partie droite de la règle
    std::vector<BridgePort *> ports;

    void  mySortPorts(std::vector<BridgePort *>  &, const float, tlp::LayoutProperty *layout );
public:
    Bridge(PortGraph *, const tlp::node);
    virtual ~Bridge();

    PortNodeType getType();

    //iterateur tulip sur les ports du bridge
    inline tlp::Iterator<BridgePort *> *getBridgePorts() const {

        return new CopyStlIterator<BridgePort *, std::vector<BridgePort *> >(ports);
    }

    inline unsigned numportsbridge() const {return ports.size();}
    void setPositions(tlp::LayoutProperty *layout);
    void setEdgesColor(tlp::ColorProperty *color, tlp::ColorProperty *bordercolor) const;
    const tlp::node getRight(const Port::PortPtr n ) const;
    virtual void resizeCenter(tlp::SizeProperty *nodeSize, tlp::LayoutProperty *layout=NULL) const;
    void resizePorts(tlp::SizeProperty *nodeSize) const;
    void draw(tlp::SizeProperty *nodeSize, tlp::ColorProperty *color, tlp::LayoutProperty *layout, tlp::ColorProperty *bordercolor) const;

    void addPort(tlp::node source, tlp::node dest);

    static bool isBridge(tlp::node, tlp::Graph *);
    //static bool isBridge(PortNode *p, PortGraph *pg);
    static tlp::node createEmptyBridge(tlp::Graph *rule, std::string rulename);
};
#endif
