/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _PORT_H
#define _PORT_H

#include <portgraph/PortBase.h>
#include <portgraph/PorgyTlpGraphStructure.h>

#include <boost/shared_ptr.hpp>

class Port : public PortBase {

public :

    Port(const tlp::node, const PortNode *);

    typedef boost::shared_ptr<Port> PortPtr;    //no need for shared_ptr anymore?

    inline tlp::Iterator<Port::PortPtr> *getConnectedPorts() const {
        return new CopyStlIterator<Port::PortPtr, std::vector<PortPtr> >(connectedPorts);
    }
    inline std::vector <PortPtr>::const_iterator connectedPortsBegin() const {return connectedPorts.begin();}

    const PortNode *getOneConnectedPortNode() const;
    unsigned int getArity() const;

    /**
     * @brief check if two ports are equivalent
     * @param isRulePort : indicates if we have the port of a rule (VERY UGLY HACK, we should enhance the UML model)
     * @return
     */
    static bool check(const PortPtr rule_port, const PortPtr modele_port, bool exact);
    unsigned int deg() const;
    inline void connectPortTo(const PortPtr p) {connectedPorts.push_back(p);}

private:
    std::vector<PortPtr>connectedPorts;		//liste des Ports distants connectés à ce port

};

#endif
