/**
 *
 * This file is part of Porgy (https://gforge.inria.fr/projects/porgy/)
 *
 * from LaBRI, University of Bordeaux 1, Inria Bordeaux - Sud Ouest and King's College London
 *
 * Porgy is free software;  you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Porgy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this Porgy.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _PORTGRAPHRULE_H
#define _PORTGRAPHRULE_H

#include <portgraph/PortGraph.h>
#include <portgraph/PorgyConstants.h>

#include <tulip/StringCollection.h>

class PortNode;
class Bridge;

class PortGraphRule:public PortGraph {

    Bridge *bridge;

public:

    //iterators
    tlp::Iterator<PortNode *> *getMember(const PorgyConstants::RuleSide);
    /**
     * @brief getPortNodes
     * @return An iterator on the portnodes of the left- and right-hand side
     */
    virtual tlp::Iterator<PortNode *> *getPortNodes(tlp::BooleanProperty *ban=NULL) const;

    tlp::BooleanProperty *getSelectionProperty();
    tlp::Iterator<tlp::edge> *getMemberEdges(const PorgyConstants::RuleSide);
    /**
     * @brief getAllEdges
     * @return Returns edges of the left and right members
     */

    tlp::Iterator<tlp::edge> *getEdges(tlp::BooleanProperty *ban=NULL);

    /**
     * @brief getSideProperty
     * @return the property containing the side information for each node. Convinience function call the static getSideProperty function.
     */
    inline tlp::IntegerProperty *getSideProperty()const{ return getSideProperty(getGraph());   }

    inline unsigned getSide(const tlp::node &n) const {return getSideProperty()->getNodeValue(n);}
    inline unsigned getSide(const tlp::edge &e) const {return getSideProperty()->getEdgeValue(e);}

    inline Bridge * getBridge() const {return bridge;}
    static bool isMember(const tlp::node n, tlp::Graph *g, const PorgyConstants::RuleSide &side);
    static bool isMember(const tlp::edge e, tlp::Graph *g, const PorgyConstants::RuleSide &side);
    tlp::StringCollection getLayoutAlgorithm();
    /**
     * @brief getSideProperty
     * @param g
     * @return
     */
    static inline tlp::IntegerProperty* getSideProperty(tlp::Graph* g){
        return g->getProperty<tlp::IntegerProperty>(PorgyConstants::RULE_SIDE);
    }

    static std::pair<tlp::Graph *,tlp::Graph *> extractLHSRHS(tlp::Graph *g, tlp::PluginProgress *pp);
    static void cleanLHSRHS(tlp::Graph *g);

    PortGraphRule (tlp::Graph *);
    virtual ~PortGraphRule();
    tlp::node findRightEquiv(const tlp::node n) const;    

    std::list<std::string> layoutPluginsList()const;
};

#endif
